from leapp.models import MultipathConfFacts8to9, MultipathConfig8to9
from leapp.reporting import Report


def _assert_foreign_report(report):
    assert report['title'] == \
        'device-mapper-multipath now defaults to ignoring foreign devices'
    assert report['severity'] == 'info'


def _assert_allow_usb_report(report):
    assert report['title'] == \
        'device-mapper-multipath now defaults to ignoring USB devices'
    assert report['severity'] == 'info'


def _assert_invalid_regexes_report(report, paths_str):
    assert report['title'] == \
        'device-mapper-multipath no longer accepts "*" as a valid regular expression'
    assert report['severity'] == 'info'
    assert paths_str in report['summary']


def _build_config(pathname, config_dir, enable_foreign_exists, invalid_regexes_exist, allow_usb_exists):
    return MultipathConfig8to9(
        pathname=pathname,
        config_dir=config_dir,
        enable_foreign_exists=enable_foreign_exists,
        invalid_regexes_exist=invalid_regexes_exist,
        allow_usb_exists=allow_usb_exists,
    )


def _build_facts(confs):
    return MultipathConfFacts8to9(configs=confs)


def test_need_everything(current_actor_context):
    config = _build_config('need_everything.conf', None, False, True, False)
    facts = _build_facts([config])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 3
    _assert_foreign_report(reports[0].report)
    _assert_allow_usb_report(reports[1].report)
    _assert_invalid_regexes_report(reports[2].report, 'need_everything.conf')


def test_need_nothing(current_actor_context):
    config = _build_config('need_nothing.conf', '/etc/multipath/conf.d', True, False, True)
    facts = _build_facts([config])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = current_actor_context.consume(Report)
    assert not reports


def test_need_foreign(current_actor_context):
    config = _build_config('need_foreign.conf', None, False, False, True)
    facts = _build_facts([config])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 1
    _assert_foreign_report(reports[0].report)


def test_need_allos_usb(current_actor_context):
    config = _build_config('need_allow_usb.conf', None, True, False, False)
    facts = _build_facts([config])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 1
    _assert_allow_usb_report(reports[0].report)


def test_invalid_regexes(current_actor_context):
    config1 = _build_config('invalid_regexes1.conf', None, True, True, True)
    config2 = _build_config('no_invalid_regexes.conf', None, True, False, True)
    config3 = _build_config('invalid_regexes2.conf', None, True, True, True)
    facts = _build_facts([config1, config2, config3])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 1
    _assert_invalid_regexes_report(reports[0].report, 'invalid_regexes1.conf and invalid_regexes2.conf')


def test_not_in_main_conf(current_actor_context):
    main_conf = _build_config('main.conf', '/etc/multipath/conf.d', False, True, False)
    other_conf = _build_config('other.conf', None, True, False, True)
    facts = _build_facts([main_conf, other_conf])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 1
    _assert_invalid_regexes_report(reports[0].report, 'main.conf')


def test_in_main_conf(current_actor_context):
    main_conf = _build_config('main.conf', '/etc/multipath/conf.d', True, True, True)
    other_conf = _build_config('other.conf', None, False, False, False)
    next_conf = _build_config('next.conf', None, False, True, False)
    last_conf = _build_config('last.conf', None, False, True, False)
    facts = _build_facts([main_conf, other_conf, next_conf, last_conf])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 1
    _assert_invalid_regexes_report(reports[0].report, 'main.conf, next.conf and last.conf')


def test_in_none_conf(current_actor_context):
    main_conf = _build_config('main.conf', '/etc/multipath/conf.d', False, False, False)
    other_conf = _build_config('other.conf', None, False, False, False)
    facts = _build_facts([main_conf, other_conf])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 2
    _assert_foreign_report(reports[0].report)
    _assert_allow_usb_report(reports[1].report)


def test_mixed_conf(current_actor_context):
    main_conf = _build_config('main.conf', None, True, False, False)
    next_conf = _build_config('next.conf', None, False, True, False)
    last_conf = _build_config('last.conf', None, True, False, False)
    facts = _build_facts([main_conf, next_conf, last_conf])
    current_actor_context.feed(facts)
    current_actor_context.run()
    reports = list(current_actor_context.consume(Report))
    assert reports and len(reports) == 2
    _assert_allow_usb_report(reports[0].report)
    _assert_invalid_regexes_report(reports[1].report, 'next.conf')



import os

import pytest

from leapp.libraries.actor import multipathconfread
from leapp.models import MultipathConfFacts, MultipathConfig, MultipathConfigOption

# TODO [Artem] We shouldn't chdir in tests
TEST_DIR = os.path.dirname(os.path.abspath(__file__))


@pytest.fixture
def adjust_cwd():
    previous_cwd = os.getcwd()
    os.chdir(TEST_DIR)
    yield
    os.chdir(previous_cwd)


CUR_DIR = ""


def build_config(val):
    all_devs_options_val = []
    for name_val, value_val in val[16]:
        option = MultipathConfigOption(name=name_val, value=value_val)
        all_devs_options_val.append(option)
    return MultipathConfig(
            pathname=val[0],
            default_path_checker=val[1],
            config_dir=val[2],
            default_retain_hwhandler=val[3],
            default_detect_prio=val[4],
            default_detect_checker=val[5],
            reassign_maps=val[6],
            hw_str_match_exists=val[7],
            ignore_new_boot_devs_exists=val[8],
            new_bindings_in_boot_exists=val[9],
            unpriv_sgio_exists=val[10],
            detect_path_checker_exists=val[11],
            overrides_hwhandler_exists=val[12],
            overrides_pg_timeout_exists=val[13],
            queue_if_no_path_exists=val[14],
            all_devs_section_exists=val[15],
            all_devs_options=all_devs_options_val)


default_rhel7_conf = build_config(
    [os.path.join(CUR_DIR, 'files/default_rhel7.conf'), 'directio', os.path.join(CUR_DIR, 'files/conf.d'), False,
     False, False, True, True, True, True, True, True, False, False, True, False, [], ])

all_devs_conf = build_config(
    [os.path.join(CUR_DIR, 'files/conf.d/all_devs.conf'), None, None, None, None, None, None, False, False, False,
     True, True, False, False, True, True,
     [('path_checker', 'rdac'), ('detect_checker', 'yes'), ('features', '2 pg_init_retries 50'),
      ('path_selector', 'service-time 0'), ('fast_io_fail_tmo', '5'), ('no_path_retry', 'queue'), ], ])

empty_conf = build_config(
    [os.path.join(CUR_DIR, 'files/conf.d/empty.conf'), None, None, None, None, None, None, False, False, False, False,
     False, False, False, False, False, [], ])

default_rhel8_conf = build_config(
    [os.path.join(CUR_DIR, 'files/default_rhel8.conf'), 'tur', '/etc/multipath/conf.d', True, True, None, False, False,
     False, False, False, False, False, False, False, False, [], ])

all_the_things_conf = build_config(
    [os.path.join(CUR_DIR, 'files/all_the_things.conf'), 'directio', os.path.join(CUR_DIR, 'files/conf.d'), False,
     False, False, True, True, True, True, True, True, True, True, True, True,
     [('no_path_retry', 'fail'), ('features', '0')], ])

already_updated_conf = build_config(
    [os.path.join(CUR_DIR, 'files/already_updated.conf'), None, os.path.join(CUR_DIR, 'files/conf.d'), None, None,
     None, None, False, False, False, False, False, False, False, False, False, [], ])

ugly1_conf = build_config(
    [os.path.join(CUR_DIR, 'files/ugly1.conf'), 'directio', os.path.join(CUR_DIR, 'files/conf.d'), False, False, False,
     True, True, True, True, True, True, True, True, True, True,
     [('dev_loss_tmo', '60'), ('path_selector', 'service-time 0')], ])

# same results as all_devs_conf
ugly2_conf = build_config(
    [os.path.join(CUR_DIR, 'files/ugly2.conf'), None, None, None, None, None, None, False, False, False, True, True,
     False, False, True, True,
     [('path_checker', 'rdac'), ('detect_checker', 'yes'), ('features', '2 pg_init_retries 50'),
      ('path_selector', 'service-time 0'), ('fast_io_fail_tmo', '5'), ('no_path_retry', 'queue'), ], ])

just_checker_conf = build_config(
    [os.path.join(CUR_DIR, 'files/just_checker.conf'), 'rdac', '/etc/multipath/conf.d', True, True, None, False, False,
     False, False, False, False, False, False, False, False, [], ])

just_detect_conf = build_config(
    [os.path.join(CUR_DIR, 'files/just_detect.conf'), None, None, None, False, None, None, False, False, False, False,
     False, False, False, False, False, [], ])

just_reassign_conf = build_config(
    [os.path.join(CUR_DIR, 'files/just_reassign.conf'), None, None, None, None, None, True, False, False, False, False,
     False, False, False, False, False, [], ])

just_exists_conf = build_config(
    [os.path.join(CUR_DIR, 'files/just_exists.conf'), None, None, None, None, None, None, False, False, False, False,
     False, False, False, True, False, [], ])

just_all_devs_conf = build_config(
    [os.path.join(CUR_DIR, 'files/just_all_devs.conf'), None, None, None, None, None, None, False, False, False, False,
     False, False, False, False, True, [], ])


def assert_config(config, expected):
    assert config.pathname == expected.pathname
    assert config.default_path_checker == expected.default_path_checker
    assert config.config_dir == expected.config_dir
    assert config.default_retain_hwhandler == expected.default_retain_hwhandler
    assert config.default_detect_prio == expected.default_detect_prio
    assert config.default_detect_checker == expected.default_detect_checker
    assert config.reassign_maps == expected.reassign_maps
    assert config.hw_str_match_exists == expected.hw_str_match_exists
    assert config.ignore_new_boot_devs_exists == expected.ignore_new_boot_devs_exists
    assert config.new_bindings_in_boot_exists == expected.new_bindings_in_boot_exists
    assert config.unpriv_sgio_exists == expected.unpriv_sgio_exists
    assert config.detect_path_checker_exists == expected.detect_path_checker_exists
    assert config.overrides_hwhandler_exists == expected.overrides_hwhandler_exists
    assert config.overrides_pg_timeout_exists == expected.overrides_pg_timeout_exists
    assert config.queue_if_no_path_exists == expected.queue_if_no_path_exists
    assert config.all_devs_section_exists == expected.all_devs_section_exists
    assert len(config.all_devs_options) == len(expected.all_devs_options)
    for i in range(len(config.all_devs_options)):
        conf_opt = config.all_devs_options[i]
        expt_opt = expected.all_devs_options[i]
        assert conf_opt.name == expt_opt.name
        assert conf_opt.value == expt_opt.value


def test_config_dir(adjust_cwd):
    expected_configs = (default_rhel7_conf, all_devs_conf, empty_conf)
    facts = multipathconfread.get_multipath_conf_facts(config_file=os.path.join(CUR_DIR, 'files/default_rhel7.conf'))
    assert facts
    assert len(facts.configs) == 3
    for i in range(len(facts.configs)):
        assert_config(facts.configs[i], expected_configs[i])


def test_already_rhel8(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/default_rhel8.conf'))
    assert config
    assert_config(config, default_rhel8_conf)


def test_all_the_things(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/all_the_things.conf'))
    assert config
    assert_config(config, all_the_things_conf)


def test_already_updated(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/already_updated.conf'))
    assert config
    assert_config(config, already_updated_conf)


def tests_ugly1(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/ugly1.conf'))
    assert config
    assert_config(config, ugly1_conf)


def tests_ugly2(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/ugly2.conf'))
    assert config
    assert_config(config, ugly2_conf)


def tests_just_checker(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/just_checker.conf'))
    assert config
    assert_config(config, just_checker_conf)


def tests_just_detect(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/just_detect.conf'))
    assert config
    assert_config(config, just_detect_conf)


def tests_just_reassign(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/just_reassign.conf'))
    assert config
    assert_config(config, just_reassign_conf)


def tests_just_exists(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/just_exists.conf'))
    assert config
    assert_config(config, just_exists_conf)


def tests_just_all_devs(adjust_cwd):
    config = multipathconfread._parse_config(os.path.join(CUR_DIR, 'files/just_all_devs.conf'))
    assert config
    assert_config(config, just_all_devs_conf)



from leapp.actors import Actor
from leapp.libraries.actor import checkdddd
from leapp.models import DetectedDeviceOrDriver, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckDetectedDevicesAndDrivers(Actor):
    """
    Checks whether or not detected devices and drivers are usable on the target system.

    In case a driver is no longer present in the target system, an inhibitor will be raised.
    If the device or driver is not maintained anymore, a warning report will be generated.
    """

    name = 'check_detected_devices_and_drivers'
    consumes = (DetectedDeviceOrDriver,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        checkdddd.process()



from leapp.actors import Actor
from leapp.libraries.actor import cupsscanner
from leapp.models import CupsChangedFeatures, DistributionSignedRPM, Report
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class CupsScanner(Actor):
    """
    Gather facts about CUPS features which needs to be migrated

    Actor checks if cups package is installed and if one or more following
    situations appears in configuration files:
    - interface scripts
    - use of 'Digest' or 'BasicDigest' authentication
    - use of 'Include' directive
    - use of 'ServerCertificate' and 'ServerKey' directives
    - use of 'SetEnv' or 'PassEnv' directives
    - use of 'PrintcapFormat' directive

    The actor creates list from gathered data.
    """

    name = 'cups_scanner'
    consumes = (DistributionSignedRPM,)
    produces = (Report, CupsChangedFeatures)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        cupsscanner.find_features()



from leapp import reporting
from leapp.actors import Actor
from leapp.models import QuaggaToFrrFacts, Report
from leapp.reporting import create_report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

COMMON_REPORT_TAGS = [
    reporting.Groups.NETWORK,
    reporting.Groups.SERVICES
]


class QuaggaReport(Actor):
    """
    Checking for babeld on RHEL-7.

    This actor is supposed to report that babeld was used on RHEL-7
    and it is no longer available in RHEL-8.
    """

    name = 'quagga_report'
    consumes = (QuaggaToFrrFacts, )
    produces = (Report, )
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        try:
            quagga_facts = next(self.consume(QuaggaToFrrFacts))
        except StopIteration:
            return
        if 'babeld' in quagga_facts.active_daemons or 'babeld' in quagga_facts.enabled_daemons:
            create_report([
                reporting.Title('Babeld is not available in FRR'),
                reporting.ExternalLink(
                    url='https://red.ht/rhel-8-configuring-routing-protocols',
                    title='Setting routing protocols in RHEL8'),
                reporting.Summary(
                    'babeld daemon which was a part of quagga implementation in RHEL7 '
                    'is not available in RHEL8 in FRR due to licensing issues.'
                ),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups(COMMON_REPORT_TAGS),
                reporting.Groups([reporting.Groups.INHIBITOR]),
                reporting.Remediation(hint='Please use RIP, OSPF or EIGRP instead of Babel')
            ])
        else:
            self.log.debug('babeld not used, moving on.')



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor.setpermissiveselinux import selinux_set_permissive
from leapp.models import SelinuxPermissiveDecision
from leapp.reporting import create_report, Report
from leapp.tags import FinalizationPhaseTag, IPUWorkflowTag


class SetPermissiveSelinux(Actor):
    """
    Set SELinux mode.

    In order to proceed with Upgrade process, SELinux should be set into permissive mode if it was
    in enforcing mode.
    """

    name = 'set_permissive_se_linux'
    consumes = (SelinuxPermissiveDecision,)
    produces = (Report,)
    tags = (FinalizationPhaseTag, IPUWorkflowTag)

    def process(self):
        for decision in self.consume(SelinuxPermissiveDecision):
            if decision.set_permissive:
                success, err_msg = selinux_set_permissive()
                if not success:
                    # FIXME: add an "action required" flag later
                    create_report([
                        reporting.Title('Could not set SElinux into permissive mode'),
                        reporting.Summary('{}'.format(err_msg)),
                        reporting.Severity(reporting.Severity.HIGH),
                        reporting.Groups([reporting.Groups.SELINUX, reporting.Groups.SECURITY]),
                        reporting.Groups([reporting.Groups.FAILURE]),
                        reporting.RelatedResource('file', '/etc/selinux/config')
                    ])
                    self.log.critical('Could not set SElinux into permissive mode: %s.' % err_msg)



from leapp.actors import Actor
from leapp.libraries.actor import targetinitramfsgenerator
from leapp.models import InitrdIncludes  # deprecated
from leapp.models import InstalledTargetKernelInfo, TargetInitramfsTasks
from leapp.tags import FinalizationPhaseTag, IPUWorkflowTag
from leapp.utils.deprecation import suppress_deprecation


@suppress_deprecation(InitrdIncludes)
class TargetInitramfsGenerator(Actor):
    """
    Regenerate the target RHEL major version initrd and include files produced by other actors
    """

    name = 'target_initramfs_generator'
    consumes = (InitrdIncludes, InstalledTargetKernelInfo, TargetInitramfsTasks)
    produces = ()
    tags = (FinalizationPhaseTag, IPUWorkflowTag)

    def process(self):
        targetinitramfsgenerator.process()



from leapp import reporting
from leapp.libraries.common.config import get_all_envs
from leapp.reporting import create_report

DEPRECATED_VARS = ['LEAPP_GRUB_DEVICE']


def process():

    vars_to_report = []

    for var in get_all_envs():
        if var.name in DEPRECATED_VARS:
            vars_to_report.append(var.name)

    if vars_to_report:
        vars_str = ' '.join(vars_to_report)
        create_report([
            reporting.Title('Leapp detected removed environment variable usage'),
            reporting.Summary('The following Leapp related environment variable was removed: ' + vars_str),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Remediation(hint='Please do not use the reported variables'),
            reporting.Groups(['inhibitor']),
            reporting.Groups([reporting.Groups.UPGRADE_PROCESS]),
        ])



import pytest

from leapp import reporting
from leapp.libraries.actor.checkfstabmountorder import (
    _get_common_path,
    _get_overshadowing_mount_points,
    check_fstab_mount_order
)
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import FstabEntry, MountEntry, StorageInfo

VAR_ENTRY = FstabEntry(fs_spec='', fs_file='/var', fs_vfstype='',
                       fs_mntops='defaults', fs_freq='0', fs_passno='0')
VAR_DUPLICATE_ENTRY = FstabEntry(fs_spec='', fs_file='/var/', fs_vfstype='',
                                 fs_mntops='defaults', fs_freq='0', fs_passno='0')
VAR_LOG_ENTRY = FstabEntry(fs_spec='', fs_file='/var/log', fs_vfstype='',
                           fs_mntops='defaults', fs_freq='0', fs_passno='0')


@pytest.mark.parametrize(
    ('path1', 'path2', 'expected_output'),
    [
        ('', '', ''),
        ('/var', '/var', '/var'),
        ('/var/lib/leapp', '/var/lib', '/var/lib'),
        ('/var/lib/leapp', '/home', '/'),
        ('/var/lib/leapp', '/var/lib/lea', '/var/lib'),
    ]
)
def test_get_common_path(path1, path2, expected_output):
    assert _get_common_path(path1, path2) == expected_output


@pytest.mark.parametrize(
    ('fstab_entries', 'expected_output'),
    [
        (
           ['/var', '/var/log'],
           set()
        ),
        (
           ['/var', '/'],
           {'/var', '/'}
        ),
        (
           ['/var/log', '/var', '/var'],
           {'/var/log', '/var'}
        ),
        (
           ['/var/log', '/home', '/var', '/var/lib/leapp'],
           {'/var/log', '/var'}
        ),
        (
           ['/var/log', '/home', '/var/lib/leapp', '/var'],
           {'/var/log', '/var', '/var/lib/leapp'}
        ),
        (
           ['/var/log', '/home', '/var', '/var/lib/lea', '/var/lib/leapp'],
           {'/var/log', '/var'}
        ),
    ]
)
def test_get_overshadowing_mount_points(fstab_entries, expected_output):
    assert _get_overshadowing_mount_points(fstab_entries) == expected_output


@pytest.mark.parametrize(
    ('storage_info', 'should_inhibit', 'duplicates'),
    [
        (StorageInfo(fstab=[]), False, False),
        (StorageInfo(fstab=[VAR_LOG_ENTRY, VAR_ENTRY]), True, False),
        (StorageInfo(fstab=[VAR_LOG_ENTRY, VAR_ENTRY, VAR_DUPLICATE_ENTRY]), True, True),
        (StorageInfo(fstab=[VAR_ENTRY, VAR_LOG_ENTRY]), False, False),
    ]
)
def test_var_lib_leapp_non_persistent_is_detected(monkeypatch, storage_info, should_inhibit, duplicates):

    created_reports = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[storage_info]))
    monkeypatch.setattr(reporting, 'create_report', created_reports)

    check_fstab_mount_order()

    if should_inhibit:
        assert created_reports.called == 1

        if duplicates:
            assert 'Detected mount points with duplicates:' in created_reports.reports[-1]['summary']



from leapp.actors import Actor
from leapp.libraries.actor.private_firewalldcollectusedobjectnames import read_config
from leapp.models import FirewalldUsedObjectNames
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class FirewalldCollectUsedObjectNames(Actor):
    """
    This actor reads firewalld's configuration and produces Model
    FirewalldUsedObjectNames.
    """

    name = 'firewalld_collect_used_object_names'
    consumes = ()
    produces = (FirewalldUsedObjectNames,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        self.produce(read_config())



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import CustomModifications

FMT_LIST_SEPARATOR = "\n    - "


def _pretty_files(messages):
    """
    Return formatted string of discovered files from obtained CustomModifications messages.
    """
    flist = []
    for msg in messages:
        actor = ' (Actor: {})'.format(msg.actor_name) if msg.actor_name else ''
        flist.append(
            '{sep}{filename}{actor}'.format(
                sep=FMT_LIST_SEPARATOR,
                filename=msg.filename,
                actor=actor
            )
        )
    return ''.join(flist)


def _is_modified_config(msg):
    # NOTE(pstodulk):
    # We are interested just about modified files for now. Having new created config
    # files is not so much important for us right now, but in future it could
    # be changed.
    if msg.component and msg.component == 'configuration':
        return msg.type == 'modified'
    return False


def _create_report(title, summary, hint, links=None):
    report_parts = [
        reporting.Title(title),
        reporting.Summary(summary),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.UPGRADE_PROCESS]),
        reporting.RemediationHint(hint)
    ]
    if links:
        report_parts += links
    reporting.create_report(report_parts)


def check_configuration_files(msgs):
    filtered_msgs = [m for m in msgs if _is_modified_config(m)]
    if not filtered_msgs:
        return
    title = 'Detected modified configuration files in leapp configuration directories.'
    summary = (
        'We have detected that some configuration files related to leapp or'
        ' upgrade process have been modified. Some of these changes could be'
        ' intended (e.g. modified repomap.json file in case of private cloud'
        ' regions or customisations done on used Satellite server) so it is'
        ' not always needed to worry about them. However they can impact'
        ' the in-place upgrade and it is good to be aware of potential problems'
        ' or unexpected results if they are not intended.'
        '\nThe list of modified configuration files:{files}'
        .format(files=_pretty_files(filtered_msgs))
    )
    hint = (
        'If some of changes in listed configuration files have not been intended,'
        ' you can restore original files by following procedure:'
        '\n1. Remove (or back up) modified files that you want to restore.'
        '\n2. Reinstall packages which owns these files.'
    )
    _create_report(title, summary, hint)


def _is_modified_code(msg):
    if msg.component not in ['framework', 'repository']:
        return False
    return msg.type == 'modified'


def check_modified_code(msgs):
    filtered_msgs = [m for m in msgs if _is_modified_code(m)]
    if not filtered_msgs:
        return
    title = 'Detected modified files of the in-place upgrade tooling.'
    summary = (
        'We have detected that some files of the tooling processing the in-place'
        ' upgrade have been modified. Note that such modifications can be allowed'
        ' only after consultation with Red Hat - e.g. when support suggests'
        ' the change to resolve discovered problem.'
        ' If these changes have not been approved by Red Hat, the in-place upgrade'
        ' is unsupported.'
        '\nFollowing files have been modified:{files}'
        .format(files=_pretty_files(filtered_msgs))
    )
    hint = 'To restore original files reinstall related packages.'
    _create_report(title, summary, hint)


def check_custom_actors(msgs):
    filtered_msgs = [m for m in msgs if m.type == 'custom']
    if not filtered_msgs:
        return
    title = 'Detected custom leapp actors or files.'
    summary = (
        'We have detected installed custom actors or files on the system.'
        ' These can be provided e.g. by third party vendors, Red Hat consultants,'
        ' or can be created by users to customize the upgrade (e.g. to migrate'
        ' custom applications).'
        ' This is allowed and appreciated. However Red Hat is not responsible'
        ' for any issues caused by these custom leapp actors.'
        ' Note that upgrade tooling is under agile development which could'
        ' require more frequent update of custom actors.'
        '\nThe list of custom leapp actors and files:{files}'
        .format(files=_pretty_files(filtered_msgs))
    )
    hint = (
        'In case of any issues connected to custom or third party actors,'
        ' contact vendor of such actors. Also we suggest to ensure the installed'
        ' custom leapp actors are up to date, compatible with the installed'
        ' packages.'
    )
    links = [
        reporting.ExternalLink(
            url='https://red.ht/customize-rhel-upgrade',
            title='Customizing your Red Hat Enterprise Linux in-place upgrade'
        )
    ]

    _create_report(title, summary, hint, links)


def report_any_modifications():
    modifications = list(api.consume(CustomModifications))
    if not modifications:
        # no modification detected
        return
    check_custom_actors(modifications)
    check_configuration_files(modifications)
    check_modified_code(modifications)



import os
from io import StringIO

import pytest

from leapp.libraries.actor import grub2mkconfigonppc64
from leapp.libraries.common import testutils
from leapp.libraries.common.testutils import CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import DefaultGrub, DefaultGrubInfo, FirmwareFacts

CUR_DIR = os.path.dirname(os.path.abspath(__file__))


class MockedRun(object):
    def __init__(self):
        self.commands = []

    def __call__(self, cmd, *args, **kwargs):
        self.commands.append(cmd)
        return {}


@pytest.mark.parametrize('cmd_issued', [True, False])
def test_run_grub2mkconfig(monkeypatch, cmd_issued):

    grub2_cfg_bls_excerpt = (
        'fi'
        'insmod blscfg'
        'blscfg'
        '### END /etc/grub.d/10_linux ###'
    )

    grub2_cfg_non_bls_excerpt = (
        '### BEGIN /etc/grub.d/10_linux ###'
        'menuentry "Red Hat Enterprise Linux Server (3.10.0-1160.45.1.el7.x86_64) 7.9 (Maipo)"'
    )

    class _mock_open(object):
        def __init__(self, path, mode):
            input_ = grub2_cfg_non_bls_excerpt if cmd_issued else grub2_cfg_bls_excerpt
            self._fp = StringIO(input_)

        def __enter__(self):
            return self._fp

        def __exit__(self, *args, **kwargs):
            return None

    bls_cfg_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='true')]
    )

    bls_cfg_not_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='false')]
    )

    bls_cfg = bls_cfg_enabled if cmd_issued else bls_cfg_not_enabled

    arch = testutils.architecture.ARCH_PPC64LE if cmd_issued else testutils.architecture.ARCH_X86_64

    ppc64le_bare = FirmwareFacts(firmware='bios', ppc64le_opal=True)
    ppc64le_virt = FirmwareFacts(firmware='bios', ppc64le_opal=False)

    opal = ppc64le_virt if cmd_issued else ppc64le_bare

    mocked_run = MockedRun()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[bls_cfg, opal], arch=arch))
    monkeypatch.setattr(grub2mkconfigonppc64, 'run', mocked_run)
    monkeypatch.setattr(grub2mkconfigonppc64, 'open', _mock_open, False)
    grub2mkconfigonppc64.process()
    if cmd_issued:
        assert mocked_run.commands == [['grub2-mkconfig', '-o', '/boot/grub2/grub.cfg']]
    else:
        assert not mocked_run.commands



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class NISConfig(Model):
    """
    The model contains NIS packages configuration status.
    """
    topic = SystemInfoTopic

    nis_not_default_conf = fields.List(fields.String(), default=[])
    """
    List of names of NIS packages with modified default configuration.

    e.g. ["ypbind", "ypserv"]
    """



import os

from leapp.libraries.actor import scancustomrepofile
from leapp.libraries.common import repofileutils
from leapp.libraries.common.testutils import produce_mocked
from leapp.libraries.stdlib import api
from leapp.models import CustomTargetRepository, CustomTargetRepositoryFile, RepositoryData, RepositoryFile

_REPODATA = [
    RepositoryData(repoid="repo1", name="repo1name", baseurl="repo1url", enabled=True),
    RepositoryData(repoid="repo2", name="repo2name", baseurl="repo2url", enabled=False),
    RepositoryData(repoid="repo3", name="repo3name", enabled=True),
    RepositoryData(repoid="repo4", name="repo4name", mirrorlist="mirror4list", enabled=True),
]

_CUSTOM_REPOS = [
    CustomTargetRepository(repoid="repo1", name="repo1name", baseurl="repo1url", enabled=True),
    CustomTargetRepository(repoid="repo2", name="repo2name", baseurl="repo2url", enabled=False),
    CustomTargetRepository(repoid="repo3", name="repo3name", baseurl=None, enabled=True),
    CustomTargetRepository(repoid="repo4", name="repo4name", baseurl=None, enabled=True),
]

_CUSTOM_REPO_FILE_MSG = CustomTargetRepositoryFile(file=scancustomrepofile.CUSTOM_REPO_PATH)


class LoggerMocked(object):
    def __init__(self):
        self.infomsg = None
        self.debugmsg = None

    def info(self, msg):
        self.infomsg = msg

    def debug(self, msg):
        self.debugmsg = msg

    def __call__(self):
        return self


def test_no_repofile(monkeypatch):
    monkeypatch.setattr(os.path, 'isfile', lambda dummy: False)
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', LoggerMocked())
    scancustomrepofile.process()
    msg = "The {} file doesn't exist. Nothing to do.".format(scancustomrepofile.CUSTOM_REPO_PATH)
    assert api.current_logger.debugmsg == msg
    assert not api.produce.called


def test_valid_repofile_exists(monkeypatch):
    def _mocked_parse_repofile(fpath):
        return RepositoryFile(file=fpath, data=_REPODATA)
    monkeypatch.setattr(os.path, 'isfile', lambda dummy: True)
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(repofileutils, 'parse_repofile', _mocked_parse_repofile)
    monkeypatch.setattr(api, 'current_logger', LoggerMocked())
    scancustomrepofile.process()
    msg = "The {} file exists.".format(scancustomrepofile.CUSTOM_REPO_PATH)
    assert api.current_logger.infomsg == msg
    assert api.produce.called == len(_CUSTOM_REPOS) + 1
    assert _CUSTOM_REPO_FILE_MSG in api.produce.model_instances
    for crepo in _CUSTOM_REPOS:
        assert crepo in api.produce.model_instances


def test_empty_repofile_exists(monkeypatch):
    def _mocked_parse_repofile(fpath):
        return RepositoryFile(file=fpath, data=[])
    monkeypatch.setattr(os.path, 'isfile', lambda dummy: True)
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(repofileutils, 'parse_repofile', _mocked_parse_repofile)
    monkeypatch.setattr(api, 'current_logger', LoggerMocked())
    scancustomrepofile.process()
    msg = "The {} file exists.".format(scancustomrepofile.CUSTOM_REPO_PATH)
    assert api.current_logger.infomsg == msg
    assert not api.produce.called



import pytest

from leapp import reporting
from leapp.libraries.actor.checkdynamiclinkerconfiguration import (
    check_dynamic_linker_configuration,
    LD_LIBRARY_PATH_VAR,
    LD_PRELOAD_VAR
)
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import DynamicLinkerConfiguration, LDConfigFile, MainLDConfigFile

INCLUDED_CONFIG_PATHS = ['/etc/ld.so.conf.d/dyninst-x86_64.conf',
                         '/etc/ld.so.conf.d/mariadb-x86_64.conf',
                         '/custom/path/custom1.conf']


@pytest.mark.parametrize(('included_configs_modifications', 'used_variables', 'modified_lines'),
                         [
                            ([False, False, False], [], []),
                            ([True, True, True], [], []),
                            ([False, False, False], [LD_LIBRARY_PATH_VAR], []),
                            ([False, False, False], [], ['modified line 1', 'midified line 2']),
                            ([True, False, True], [LD_LIBRARY_PATH_VAR, LD_PRELOAD_VAR], ['modified line']),
                        ])
def test_check_ld_so_configuration(monkeypatch, included_configs_modifications, used_variables, modified_lines):
    assert len(INCLUDED_CONFIG_PATHS) == len(included_configs_modifications)

    main_config = MainLDConfigFile(path="/etc/ld.so.conf", modified=any(modified_lines), modified_lines=modified_lines)
    included_configs = []
    for path, modified in zip(INCLUDED_CONFIG_PATHS, included_configs_modifications):
        included_configs.append(LDConfigFile(path=path, modified=modified))

    configuration = DynamicLinkerConfiguration(main_config=main_config,
                                               included_configs=included_configs,
                                               used_variables=used_variables)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[configuration]))
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    check_dynamic_linker_configuration()

    report_expected = any(included_configs_modifications) or modified_lines or used_variables
    if not report_expected:
        assert reporting.create_report.called == 0
        return

    assert reporting.create_report.called == 1
    assert 'configuration for dynamic linker' in reporting.create_report.reports[0]['title']
    summary = reporting.create_report.reports[0]['summary']

    if any(included_configs_modifications):
        assert 'The following drop in config files were marked as custom:' in summary
    for config, modified in zip(INCLUDED_CONFIG_PATHS, included_configs_modifications):
        assert modified == (config in summary)

    if modified_lines:
        assert 'The /etc/ld.so.conf file has unexpected contents' in summary
    for line in modified_lines:
        assert line in summary

    if used_variables:
        assert 'The following variables contain unexpected dynamic linker configuration:' in summary
    for var in used_variables:
        assert '- {}'.format(var) in summary



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class KernelModuleParameter(Model):
    topic = SystemFactsTopic

    name = fields.String()
    value = fields.String()


class ActiveKernelModule(Model):
    topic = SystemFactsTopic

    filename = fields.String()
    parameters = fields.List(fields.Model(KernelModuleParameter))
    signature = fields.Nullable(fields.String())


class ActiveKernelModulesFacts(Model):
    topic = SystemFactsTopic

    kernel_modules = fields.List(fields.Model(ActiveKernelModule))



import pytest

from leapp import reporting
from leapp.libraries.actor import checkinsightsautoregister
from leapp.libraries.common import rhsm
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api


@pytest.mark.parametrize(
    ("skip_rhsm", "no_register", "should_report"),
    [
        (False, False, True),
        (False, True, False),
        (True, False, False),
        (True, True, False),
    ],
)
def test_should_report(monkeypatch, skip_rhsm, no_register, should_report):

    monkeypatch.setattr(rhsm, "skip_rhsm", lambda: skip_rhsm)
    monkeypatch.setattr(
        api,
        "current_actor",
        CurrentActorMocked(
            envars={"LEAPP_NO_INSIGHTS_REGISTER": "1" if no_register else "0"}
        ),
    )

    def ensure_package_mocked(package):
        assert package == checkinsightsautoregister.INSIGHTS_CLIENT_PKG
        return False

    monkeypatch.setattr(
        checkinsightsautoregister, "_ensure_package", ensure_package_mocked
    )

    called = [False]

    def _report_registration_info_mocked(_):
        called[0] = True

    monkeypatch.setattr(
        checkinsightsautoregister,
        "_report_registration_info",
        _report_registration_info_mocked,
    )

    checkinsightsautoregister.process()

    assert called[0] == should_report


@pytest.mark.parametrize(
    "already_installed, should_install", [(True, False), (False, True)]
)
def test_install_task_produced(monkeypatch, already_installed, should_install):

    def has_package_mocked(*args, **kwargs):
        return already_installed

    monkeypatch.setattr(checkinsightsautoregister, "has_package", has_package_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    checkinsightsautoregister._ensure_package(
        checkinsightsautoregister.INSIGHTS_CLIENT_PKG
    )

    assert api.produce.called == should_install


@pytest.mark.parametrize("installing_client", (True, False))
def test_report_created(monkeypatch, installing_client):

    created_reports = create_report_mocked()
    monkeypatch.setattr(reporting, "create_report", created_reports)

    checkinsightsautoregister._report_registration_info(installing_client)

    assert created_reports.called



from leapp.actors import Actor
from leapp.libraries.actor import persistentnetnamesconfig
from leapp.models import (
    InitrdIncludes,
    PersistentNetNamesFacts,
    PersistentNetNamesFactsInitramfs,
    RenamedInterfaces,
    TargetInitramfsTasks
)
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag
from leapp.utils.deprecation import suppress_deprecation


@suppress_deprecation(InitrdIncludes)
class PersistentNetNamesConfig(Actor):
    """
    Generate udev persistent network naming configuration

    This actor generates systemd-udevd link files for each physical ethernet interface present on RHEL-7
    in case we notice that interface name differs on RHEL-8. Link file configuration will assign RHEL-7 version of
    a name. Actors produces list of interfaces which changed name between RHEL-7 and RHEL-8.
    """

    name = 'persistentnetnamesconfig'
    consumes = (PersistentNetNamesFacts, PersistentNetNamesFactsInitramfs)
    produces = (RenamedInterfaces, InitrdIncludes, TargetInitramfsTasks)
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)
    initrd_files = []

    def process(self):
        persistentnetnamesconfig.process()



import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import TargetOSInstallationImage, TargetUserSpaceInfo


def mount_target_iso():
    target_os_iso = next(api.consume(TargetOSInstallationImage), None)
    target_userspace_info = next(api.consume(TargetUserSpaceInfo), None)

    if not target_os_iso:
        return

    mountpoint = os.path.join(target_userspace_info.path, target_os_iso.mountpoint[1:])
    if not os.path.exists(mountpoint):
        # The target userspace container exists, however, the mountpoint has been removed during cleanup.
        os.makedirs(mountpoint)
    try:
        run(['mount', target_os_iso.path, mountpoint])
    except CalledProcessError as err:
        # Unlikely, since we are checking that the ISO is mountable and located on a persistent partition. This would
        # likely mean that either the fstab entry for the partition points uses a different device that the one that
        # was mounted during pre-reboot, or the fstab has been tampered with before rebooting. Either way, there is
        # nothing at this point how we can recover.
        msg = 'Failed to mount the target RHEL ISO file containing RPMs to install during the upgrade.'
        raise StopActorExecutionError(message=msg, details={'details': '{0}'.format(err)})



import os

from leapp.libraries.common import config
from leapp.libraries.common.config.version import get_source_major_version, get_target_major_version
from leapp.libraries.stdlib import api, run
from leapp.models import GpgKey

GPG_CERTS_FOLDER = 'rpm-gpg'


def get_pubkeys_from_rpms(installed_rpms):
    """
    Return the list of fingerprints of GPG keys in RPM DB

    This function returns short 8 characters fingerprints of trusted GPG keys
    "installed" in the source OS RPM database. These look like normal packages
    named "gpg-pubkey" and the fingerprint is present in the version field.

    :param installed_rpms: List of installed RPMs
    :type installed_rpms: list(leapp.models.RPM)
    :return: list of GPG keys from RPM DB
    :rtype: list(leapp.models.GpgKey)
    """
    return [GpgKey(fingerprint=pkg.version, rpmdb=True) for pkg in installed_rpms.items if pkg.name == 'gpg-pubkey']


def _gpg_show_keys(key_path):
    """
    Show keys in given file in version-agnostic manner

    This runs gpg --show-keys (EL8) or gpg --with-fingerprints (EL7)
    to verify the given file exists, is readable and contains valid
    OpenPGP key data, which is printed in parsable format (--with-colons).
    """
    try:
        cmd = ['gpg2']
        # RHEL7 gnupg requires different switches to get the same output
        if get_source_major_version() == '7':
            cmd.append('--with-fingerprint')
        else:
            cmd.append('--show-keys')
        cmd += ['--with-colons', key_path]
        # TODO: discussed, most likely the checked=False will be dropped
        # and error will be handled in other functions
        return run(cmd, split=True, checked=False)
    except OSError as err:
        # NOTE: this is hypothetic; gnupg2 has to be installed on RHEL 7+
        error = 'Failed to read fingerprint from GPG key {}: {}'.format(key_path, str(err))
        api.current_logger().error(error)
        return {}


def _parse_fp_from_gpg(output):
    """
    Parse the output of gpg --show-keys --with-colons.

    Return list of 8 characters fingerprints per each gpgkey for the given
    output from stdlib.run() or None if some error occurred. Either the
    command return non-zero exit code, the file does not exists, its not
    readable or does not contain any openpgp data.
    """
    if not output or output['exit_code']:
        return []

    # we are interested in the lines of the output starting with "pub:"
    # the colons are used for separating the fields in output like this
    # pub:-:4096:1:999F7CBF38AB71F4:1612983048:::-:::escESC::::::23::0:
    #              ^--------------^ this is the fingerprint we need
    #                      ^------^ but RPM version is just the last 8 chars lowercase
    # Also multiple gpg keys can be stored in the file, so go through all "pub"
    # lines
    gpg_fps = []
    for line in output['stdout']:
        if not line or not line.startswith('pub:'):
            continue
        parts = line.split(':')
        if len(parts) >= 4 and len(parts[4]) == 16:
            gpg_fps.append(parts[4][8:].lower())
        else:
            api.current_logger().warning(
                'Cannot parse the gpg2 output. Line: "{}"'
                .format(line)
            )

    return gpg_fps


def get_gpg_fp_from_file(key_path):
    """
    Return the list of public key fingerprints from the given file

    Log warning in case no OpenPGP data found in the given file or it is not
    readable for some reason.

    :param key_path: Path to the file with GPG key(s)
    :type key_path: str
    :return: List of public key fingerprints from the given file
    :rtype: list(str)
    """
    res = _gpg_show_keys(key_path)
    fp = _parse_fp_from_gpg(res)
    if not fp:
        error_msg = 'Unable to read OpenPGP keys from {}: {}'.format(key_path, res['stderr'])
        api.current_logger().warning(error_msg)
    return fp


def get_path_to_gpg_certs():
    """
    Get path to the directory with trusted target gpg keys in the common leapp repository.

    GPG keys stored under this directory are considered as trusted and are
    installed during the upgrade process.

    :return: Path to the directory with GPG keys stored under the common leapp repository.
    :rtype: str
    """
    target_major_version = get_target_major_version()
    target_product_type = config.get_product_type('target')
    certs_dir = target_major_version
    # only beta is special in regards to the GPG signing keys
    if target_product_type == 'beta':
        certs_dir = '{}beta'.format(target_major_version)
    return os.path.join(api.get_common_folder_path(GPG_CERTS_FOLDER), certs_dir)


def is_nogpgcheck_set():
    """
    Return True if the GPG check should be skipped.

    The GPG check is skipped if leapp is executed with LEAPP_NOGPGCHECK=1
    or with the --nogpgcheck CLI option. In both cases, actors will see
    LEAPP_NOGPGCHECK is '1'.

    :rtype: bool
    """
    return config.get_env('LEAPP_NOGPGCHECK', False) == '1'



import os

import pytest

from leapp.exceptions import StopActorExecution
from leapp.libraries.common import grub, mdraid
from leapp.libraries.common.testutils import logger_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import DefaultGrub, DefaultGrubInfo
from leapp.utils.deprecation import suppress_deprecation

BOOT_PARTITION = '/dev/vda1'
BOOT_DEVICE = '/dev/vda'

MD_BOOT_DEVICE = '/dev/md0'
MD_BOOT_DEVICES_WITH_GRUB = ['/dev/sda', '/dev/sdb']

VALID_DD = b'GRUB GeomHard DiskRead Error'
INVALID_DD = b'Nothing to see here!'

CUR_DIR = os.path.dirname(os.path.abspath(__file__))


def raise_call_error(args=None):
    raise CalledProcessError(
        message='A Leapp Command Error occurred.',
        command=args,
        result={'signal': None, 'exit_code': 1, 'pid': 0, 'stdout': 'fake', 'stderr': 'fake'}
    )


class RunMocked(object):

    def __init__(self, raise_err=False, boot_on_raid=False):
        self.called = 0
        self.args = None
        self.raise_err = raise_err
        self.boot_on_raid = boot_on_raid

    def __call__(self, args, encoding=None):
        self.called += 1
        self.args = args
        if self.raise_err:
            raise_call_error(args)

        if self.args == ['grub2-probe', '--target=device', '/boot']:
            stdout = MD_BOOT_DEVICE if self.boot_on_raid else BOOT_PARTITION

        elif self.args == ['lsblk', '-spnlo', 'name', BOOT_PARTITION]:
            stdout = BOOT_DEVICE
        elif self.args[:-1] == ['lsblk', '-spnlo', 'name']:
            stdout = self.args[-1][:-1]

        return {'stdout': stdout}


def open_mocked(fn, flags):
    if fn == BOOT_DEVICE or fn in MD_BOOT_DEVICES_WITH_GRUB:
        path = os.path.join(CUR_DIR, 'grub_valid')
    else:
        path = os.path.join(CUR_DIR, 'grub_invalid')
    return open(path, 'r')


def open_invalid(fn, flags):
    return open(os.path.join(CUR_DIR, 'grub_invalid'), 'r')


def read_mocked(f, size):
    return f.read(size)


def close_mocked(f):
    f.close()


@suppress_deprecation(grub.get_grub_device)
def test_get_grub_device_library(monkeypatch):
    run_mocked = RunMocked()
    monkeypatch.setattr(grub, 'run', run_mocked)
    monkeypatch.setattr(os, 'open', open_mocked)
    monkeypatch.setattr(os, 'read', read_mocked)
    monkeypatch.setattr(os, 'close', close_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    result = grub.get_grub_device()
    assert grub.run.called == 2
    assert BOOT_DEVICE == result
    assert not api.current_logger.warnmsg
    assert 'GRUB is installed on {}'.format(result) in api.current_logger.infomsg


@suppress_deprecation(grub.get_grub_device)
def test_get_grub_device_fail_library(monkeypatch):
    # TODO(pstodulk): cover here also case with OSError (covered now in actors,
    # so keeping for the future when we have a time)
    run_mocked = RunMocked(raise_err=True)
    monkeypatch.setattr(grub, 'run', run_mocked)
    monkeypatch.setattr(os, 'open', open_mocked)
    monkeypatch.setattr(os, 'read', read_mocked)
    monkeypatch.setattr(os, 'close', close_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    with pytest.raises(StopActorExecution):
        grub.get_grub_device()
    assert grub.run.called == 1
    err = 'Could not get name of underlying /boot partition'
    assert err in api.current_logger.warnmsg


@suppress_deprecation(grub.get_grub_device)
def test_device_no_grub_library(monkeypatch):
    run_mocked = RunMocked()
    monkeypatch.setattr(grub, 'run', run_mocked)
    monkeypatch.setattr(os, 'open', open_invalid)
    monkeypatch.setattr(os, 'read', read_mocked)
    monkeypatch.setattr(os, 'close', close_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    result = grub.get_grub_device()
    assert grub.run.called == 2
    assert not result


@pytest.mark.parametrize('enabled', [True, False])
def test_is_blscfg_library(monkeypatch, enabled):
    bls_cfg_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='true')]
    )

    bls_cfg_not_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='false')]
    )

    bls_cfg = bls_cfg_enabled if enabled else bls_cfg_not_enabled

    result = grub.is_blscfg_enabled_in_defaultgrub(bls_cfg)
    if enabled:
        assert result
    else:
        assert not result


def is_mdraid_dev_mocked(dev):
    return dev == '/dev/md0'


def test_get_grub_devices_one_device(monkeypatch):
    run_mocked = RunMocked()
    monkeypatch.setattr(grub, 'run', run_mocked)
    monkeypatch.setattr(os, 'open', open_mocked)
    monkeypatch.setattr(os, 'read', read_mocked)
    monkeypatch.setattr(os, 'close', close_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(mdraid, 'is_mdraid_dev', is_mdraid_dev_mocked)

    result = grub.get_grub_devices()
    assert grub.run.called == 2
    assert [BOOT_DEVICE] == result
    assert not api.current_logger.warnmsg
    assert 'GRUB is installed on {}'.format(",".join(result)) in api.current_logger.infomsg


@pytest.mark.parametrize(
    ',component_devs,expected',
    [
        (['/dev/sda1', '/dev/sdb1'], MD_BOOT_DEVICES_WITH_GRUB),
        (['/dev/sda1', '/dev/sdb1', '/dev/sdc1', '/dev/sdd1'], MD_BOOT_DEVICES_WITH_GRUB),
        (['/dev/sda2', '/dev/sdc1'], ['/dev/sda']),
        (['/dev/sdd3', '/dev/sdb2'], ['/dev/sdb']),
    ]
)
def test_get_grub_devices_raid_device(monkeypatch, component_devs, expected):
    run_mocked = RunMocked(boot_on_raid=True)
    monkeypatch.setattr(grub, 'run', run_mocked)
    monkeypatch.setattr(os, 'open', open_mocked)
    monkeypatch.setattr(os, 'read', read_mocked)
    monkeypatch.setattr(os, 'close', close_mocked)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(mdraid, 'is_mdraid_dev', is_mdraid_dev_mocked)

    def get_component_devices_mocked(raid_dev):
        assert raid_dev == MD_BOOT_DEVICE
        return component_devs

    monkeypatch.setattr(mdraid, 'get_component_devices', get_component_devices_mocked)

    result = grub.get_grub_devices()
    assert grub.run.called == 1 + len(component_devs)  # grub2-probe + Nx lsblk
    assert sorted(expected) == result
    assert not api.current_logger.warnmsg
    assert 'GRUB is installed on {}'.format(",".join(result)) in api.current_logger.infomsg



from leapp.actors import Actor
from leapp.libraries.actor import repairsystemdsymlinks
from leapp.models import SystemdBrokenSymlinksSource, SystemdBrokenSymlinksTarget, SystemdServicesInfoSource
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class RepairSystemdSymlinks(Actor):
    """
    Fix broken or incorrect systemd symlinks

    Symlinks are handled in the following fashion, if the symlink points to:
        - a removed unit, such a symlink is deleted
        - a unit whose installation has been changed (e.g. changed WantedBy),
          such symlinks are fixed (re-enabled using systemctl)

    Symlinks that have been already broken before the in-place upgrade are ignored.
    """

    name = 'repair_systemd_symlinks'
    consumes = (SystemdBrokenSymlinksSource, SystemdBrokenSymlinksTarget, SystemdServicesInfoSource)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        repairsystemdsymlinks.process()



from leapp.actors import Actor
from leapp.libraries.actor.pes_events_scanner import process
from leapp.models import (
    ConsumedDataAsset,
    DistributionSignedRPM,
    EnabledModules,
    PESRpmTransactionTasks,
    RepositoriesBlacklisted,
    RepositoriesFacts,
    RepositoriesMapping,
    RepositoriesSetupTasks,
    RHUIInfo,
    RpmTransactionTasks
)
from leapp.reporting import Report
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class PesEventsScanner(Actor):
    """
    Provides data about package events from Package Evolution Service.

    After collecting data from a provided JSON file containing Package Evolution Service events, a
    message with relevant data will be produced to help DNF Upgrade transaction calculation.
    """

    name = 'pes_events_scanner'
    consumes = (
        EnabledModules,
        DistributionSignedRPM,
        RepositoriesBlacklisted,
        RepositoriesFacts,
        RepositoriesMapping,
        RHUIInfo,
        RpmTransactionTasks,
    )
    produces = (ConsumedDataAsset, PESRpmTransactionTasks, RepositoriesSetupTasks, Report)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        process()



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import IfCfg, NetworkManagerConnection

FMT_LIST_SEPARATOR = '\n    - '


def process():
    wep_files = []

    # Scan NetworkManager native keyfile connections
    for nmconn in api.consume(NetworkManagerConnection):
        for setting in nmconn.settings:
            if not setting.name == 'wifi-security':
                continue

            for prop in setting.properties:
                if not prop.name == 'key-mgmt':
                    continue
                if prop.value in ('none', 'ieee8021x'):
                    wep_files.append(nmconn.filename)

    # Scan legacy ifcfg files & secrets
    for ifcfg in api.consume(IfCfg):
        props = ifcfg.properties
        if ifcfg.secrets is not None:
            props = props + ifcfg.secrets

        for prop in props:
            name = prop.name
            value = prop.value

            # Dynamic WEP
            if name == 'KEY_MGMT' and value.upper() == 'IEEE8021X':
                wep_files.append(ifcfg.filename)
                continue

            # Static WEP, possibly with agent-owned secrets
            if name in ('KEY_PASSPHRASE1', 'KEY1', 'WEP_KEY_FLAGS'):
                wep_files.append(ifcfg.filename)
                continue

    if wep_files:
        title = 'Wireless networks using unsupported WEP encryption detected'
        summary = ('The Wired Equivalent Privacy (WEP) algorithm used for'
                   ' authenticating to wireless networks has been phased out'
                   ' due to known security weaknesses. Configuration for networks'
                   ' that use the phased out WEP algorithm is present in the system'
                   ' and will not work after the upgrade.'
                   ' Files with the problematic configuration:{}').format(
            ''.join(['{}{}'.format(FMT_LIST_SEPARATOR, bfile) for bfile in wep_files])
        )
        remediation = ('Remove configuration for networks that use WEP or'
                       ' upgrade the networks to use more secure encryption'
                       ' algorithms, such as ones defined by WPA2 or WPA3.')
        reporting.create_report([
            reporting.Title(title),
            reporting.Summary(summary),
            reporting.Remediation(hint=remediation),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.NETWORK, reporting.Groups.SERVICES]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.RelatedResource('package', 'NetworkManager-wifi'),
        ] + [
            reporting.RelatedResource('file', fname)
            for fname in wep_files
        ])



from leapp.actors import Actor
from leapp.libraries.actor.checkhacluster import check_ha_cluster
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class Checkhacluster(Actor):
    """
    Check if HA Cluster is in use. If yes, inhibit the upgrade process.

    The system is considered as part of cluster if a corosync.conf file
    (/etc/corosync/corosync.conf) can be found there.
    Also the system can be a part of a cluster as a remote node. In such case
    a cib file (/var/lib/pacemaker/cib/cib.xml) can be found there.
    """

    name = "check_ha_cluster"
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        check_ha_cluster()



from leapp import reporting
from leapp.actors import Actor
from leapp.models import XorgDrvFacts
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

SUMMARY_XORG_DEPRECATE_DRIVERS_FMT = (
    'Leapp has detected the use of some deprecated Xorg drivers. '
    'Using these drivers could lead to a broken graphical session after the upgrade. '
    'Any custom configuration related to these drivers will be ignored. '
    'The list of used deprecated drivers: {}')

SUMMARY_XORG_DEPRECATE_DRIVERS_HINT = (
    'Please uninstall the Xorg driver and remove the corresponding driver '
    'customisation entries from the X.Org configuration files and directories, '
    'such as `/etc/X11/xorg.conf` and `/etc/X11/xorg.conf.d/` and reboot before '
    'upgrading to make sure you have a graphical session after upgrading.'
)
FMT_LIST_SEPARATOR = '\n    - {}'


def _printable_drv(facts):
    output = ''
    for fact in facts:
        for driver in fact.xorg_drivers:
            output += FMT_LIST_SEPARATOR.format(driver.driver)
            if driver.has_options:
                output += ' (with custom driver options)'
    return output


class XorgDrvCheck8to9(Actor):
    """
    Warn if Xorg deprecated drivers are in use.
    """

    name = 'xorgdrvcheck8to9'
    consumes = (XorgDrvFacts,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        facts = self.consume(XorgDrvFacts)
        deprecated_drivers = _printable_drv(facts)
        if len(deprecated_drivers) > 0:
            create_report([
                reporting.Title('Deprecated Xorg driver detected'),
                reporting.Summary(SUMMARY_XORG_DEPRECATE_DRIVERS_FMT.format(deprecated_drivers)),
                reporting.Severity(reporting.Severity.MEDIUM),
                reporting.Groups([reporting.Groups.DRIVERS]),
                reporting.Remediation(hint=SUMMARY_XORG_DEPRECATE_DRIVERS_HINT)
                ])



from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api


def process(openssh_messages):
    config = next(openssh_messages, None)
    if list(openssh_messages):
        api.current_logger().warning('Unexpectedly received more than one OpenSshConfig message.')
    if not config:
        raise StopActorExecutionError(
            'Could not check openssh configuration', details={'details': 'No OpenSshConfig facts found.'}
        )

    if config.use_privilege_separation is not None and \
       config.use_privilege_separation != "sandbox":
        reporting.create_report([
            reporting.Title('OpenSSH configured not to use privilege separation sandbox'),
            reporting.Summary(
                'OpenSSH is configured to disable privilege '
                'separation sandbox, which is decreasing security '
                'and is no longer supported in RHEL 8'
            ),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.NETWORK,
                    reporting.Groups.SERVICES
            ]),
        ])



from leapp import reporting
from leapp.actors import Actor
from leapp.models import FirewalldUsedObjectNames
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class FirewalldCheckServiceTftpClient(Actor):
    """
    This actor will inhibit if firewalld's configuration is using service
    'tftp-client'.
    """

    name = 'firewalld_check_service_tftp_client'
    consumes = (FirewalldUsedObjectNames,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        tftp_client_service = 'tftp-client'
        send_report = False

        for facts in self.consume(FirewalldUsedObjectNames):
            if tftp_client_service in facts.services:
                send_report = True

        if send_report:
            create_report([
                reporting.Title('Firewalld Service tftp-client Is Unsupported'),
                reporting.Summary('Firewalld has service "{service}" enabled. '
                                  'Service "{service}" has been removed in RHEL-9.'.format(
                                      service=tftp_client_service)),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([reporting.Groups.SANITY, reporting.Groups.FIREWALL]),
                reporting.Groups([reporting.Groups.INHIBITOR]),
                reporting.Remediation(
                    hint=(
                        'Remove all usage of service "{service}" from '
                        'firewalld\'s permanent configuration. '
                        'It may be in use by: zones, policies, or rich rules.\n'
                        'Usage can be found by listing zone and policy '
                        'configuration:\n'
                        '  # firewall-cmd --permanent --list-all-zones\n'
                        '  # firewall-cmd --permanent --list-all-policies\n'
                        'Example to remove usage from a zone:\n'
                        '  # firewall-cmd --permanent --zone public '
                        ' --remove-service {service}\n'.format(
                            service=tftp_client_service)
                        )
                    ),
            ])



import os
import re

BrlttyConf = '/etc/brltty.conf'


def check_for_unsupported_cfg():
    migrate_file = None
    migrate_bt = False
    migrate_espeak = False
    regex_bt = re.compile(r'\b((bth)|(bluez))([:\-][0-9a-fA-F]{2}){6}\b')
    regex_espeak = re.compile(r'^\s*speech-driver\s+es\b')
    if os.path.exists(BrlttyConf):
        with open(BrlttyConf) as file_check:
            for line in file_check:
                if regex_bt.search(line):
                    migrate_bt = True
                if regex_espeak.search(line):
                    migrate_espeak = True
                if migrate_bt and migrate_espeak:
                    break
    migrate_file = BrlttyConf if migrate_espeak or migrate_bt else ''
    return (migrate_file, migrate_bt, migrate_espeak)



import os
from collections import namedtuple

import pytest

from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import repofileutils, rhsm
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import RepositoryData, RepositoryFile
from leapp.utils.report import is_inhibitor

Repository = namedtuple('Repository', ['repoid', 'file'])
LIST_SEPARATOR = '\n    - '

# External commands called by the RHSM library
CMD_RHSM_LIST_CONSUMED = ('subscription-manager', 'list', '--consumed')
CMD_RHSM_STATUS = ('subscription-manager', 'status')
CMD_RHSM_RELEASE = ('subscription-manager', 'release')
CMD_RHSM_LIST_ENABLED_REPOS = ('subscription-manager', 'repos', '--list-enabled')

RHSM_STATUS_OUTPUT_NOSCA = '''
+-------------------------------------------+
   System Status Details
+-------------------------------------------+
Overall Status: Current

System Purpose Status: Not Specified
'''

RHSM_STATUS_OUTPUT_SCA = '''
+-------------------------------------------+
   System Status Details
+-------------------------------------------+
Overall Status: Current

System Purpose Status: Matched

Content Access Mode is set to Simple Content Access
'''

# Used to simulate realistic output of RHSM, therefore carries more information than `Repository` namedtuple
RHSMRepositoryEntry = namedtuple('RHSMRepositoryEntry', ('id', 'name', 'url', 'enabled'))  # For clarity purposes
RHSM_ENABLED_REPOS = [
    RHSMRepositoryEntry(
        id='rhel-8-for-x86_64-appstream-rpms',
        name='Appstream',
        url='some_url',
        enabled='1'),
    RHSMRepositoryEntry(
        id='satellite-tools-6.6-for-rhel-8-x86_64-rpms',
        name='Satellite',
        url='some_url',
        enabled='1'),
    RHSMRepositoryEntry(
        id='rhel-8-for-x86_64-baseos-rpms',
        name='Base',
        url='some_url',
        enabled='1')
]


class IsolatedActionsMocked(object):
    def __init__(self, call_stdout=None, raise_err=False):
        self.commands_called = []
        self.call_return = {'stdout': call_stdout, 'stderr': None}
        self.raise_err = raise_err

        # A map from called commands to their mocked output
        self.mocked_command_call_outputs = dict()

    def call(self, cmd, *args, **dummy_kwargs):
        self.commands_called.append(cmd)
        if self.raise_err:
            raise_call_error(cmd)

        return self.mocked_command_call_outputs.get(
            tuple(cmd),  # Cast to tuple, as list is not hashable
            self.call_return)

    def add_mocked_command_call_with_stdout(self, cmd, stdout):
        # We cast `cmd` from list to tuple, as a list cannot be hashed
        self.mocked_command_call_outputs[tuple(cmd)] = {
            'stdout': stdout,
            'stderr': None}

    def full_path(self, path):
        return path


@pytest.fixture
def actor_mocked(monkeypatch):
    """
    Fixture providing a mocked actor that was already used to monkeypatch api.current_actor.

    Introduced to reduce repetition inside tests.
    """
    actor = CurrentActorMocked()
    monkeypatch.setattr(api, 'current_actor', actor)
    return actor


@pytest.fixture
def context_mocked():
    return IsolatedActionsMocked()


def raise_call_error(args=None, exit_code=1):
    raise CalledProcessError(
        message='Command {0} failed with exit code {1}.'.format(str(args), exit_code),
        command=args,
        result={'signal': None, 'exit_code': exit_code, 'pid': 0, 'stdout': 'fake out', 'stderr': 'fake err'}
    )


def _gen_repo(repoid):
    return RepositoryData(repoid=repoid, name='name {}'.format(repoid))


def _gen_repofile(rfile, data=None):
    if data is None:
        data = [_gen_repo("{}-{}".format(rfile.split("/")[-1], i)) for i in range(3)]
    return RepositoryFile(file=rfile, data=data)


@pytest.mark.parametrize('other_repofiles', [
    [],
    [_gen_repofile("foo")],
    [_gen_repofile("foo"), _gen_repofile("bar")],
])
@pytest.mark.parametrize('rhsm_repofile', [
    None,
    _gen_repofile(rhsm._DEFAULT_RHSM_REPOFILE, []),
    _gen_repofile(rhsm._DEFAULT_RHSM_REPOFILE, [_gen_repo("rh-0")]),
    _gen_repofile(rhsm._DEFAULT_RHSM_REPOFILE),
])
def test_get_available_repo_ids(monkeypatch, other_repofiles, rhsm_repofile):
    context_mocked = IsolatedActionsMocked()
    repos = other_repofiles[:]
    if rhsm_repofile:
        repos.append(rhsm_repofile)
    rhsm_repos = [repo.repoid for repo in rhsm_repofile.data] if rhsm_repofile else []

    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(rhsm, '_inhibit_on_duplicate_repos', lambda x: None)
    monkeypatch.setattr(repofileutils, 'get_parsed_repofiles', lambda x: repos)

    result = rhsm.get_available_repo_ids(context_mocked)

    rhsm_repos.sort()
    assert context_mocked.commands_called == [['yum', 'clean', 'all']]
    assert result == rhsm_repos
    if result:
        msg = (
            'The following repoids are available through RHSM:{0}{1}'
            .format(LIST_SEPARATOR, LIST_SEPARATOR.join(rhsm_repos))
        )
        assert msg in api.current_logger.infomsg
    else:
        assert 'There are no repos available through RHSM.' in api.current_logger.infomsg


def test_get_available_repo_ids_error():
    context_mocked = IsolatedActionsMocked(raise_err=True)

    with pytest.raises(StopActorExecutionError) as err:
        rhsm.get_available_repo_ids(context_mocked)

    assert 'Unable to use yum' in str(err)


def test_inhibit_on_duplicate_repos(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    repofiles = [
        _gen_repofile("foo", [_gen_repo('repoX'), _gen_repo('repoY')]),
        _gen_repofile("bar", [_gen_repo('repoX')]),
    ]

    rhsm._inhibit_on_duplicate_repos(repofiles)

    dups = ['repoX']
    assert ('The following repoids are defined multiple times:{0}{1}'
            .format(LIST_SEPARATOR, LIST_SEPARATOR.join(dups))) in api.current_logger.warnmsg
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['title'] == 'A YUM/DNF repository defined multiple times'
    summary = ('The following repositories are defined multiple times:{0}{1}'
               .format(LIST_SEPARATOR, LIST_SEPARATOR.join(dups)))
    assert summary in reporting.create_report.report_fields['summary']


def test_inhibit_on_duplicate_repos_no_dups(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    rhsm._inhibit_on_duplicate_repos([_gen_repofile("foo")])

    assert not api.current_logger.warnmsg
    assert reporting.create_report.called == 0


def test_sku_listing(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the rhsm library can obtain used SKUs correctly."""
    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_LIST_CONSUMED, 'SKU: 598339696910')

    attached_skus = rhsm.get_attached_skus(context_mocked)

    assert_fail_description = 'Some calls to subscription-manager were expected.'
    assert context_mocked.commands_called, assert_fail_description

    assert_fail_description = 'RHSM command reported 1 SKU, however {0} were detected.'.format(
        len(attached_skus)
    )
    assert len(attached_skus) == 1, assert_fail_description

    assert_fail_description = 'The parsed SKU is different than the one contained in the mocked RHSM output.'
    assert attached_skus[0] == '598339696910', assert_fail_description


def test_scanrhsminfo_with_skip_rhsm(monkeypatch, context_mocked):
    """Tests whether the scan_rhsm_info respects the LEAPP_NO_RHSM environmental variable."""
    mocked_actor = CurrentActorMocked(envars={'LEAPP_NO_RHSM': '1'})
    monkeypatch.setattr(api, 'current_actor', mocked_actor)

    result = rhsm.scan_rhsm_info(context_mocked)

    assert_fail_description = 'No external shell commands should be executed when RHSM is skipped.'
    assert not context_mocked.commands_called, assert_fail_description

    assert result is None, 'The `scan_rhsm_info` should not provide any output when RHSM is skipped.'


def test_get_release(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library correctly retrieves release from RHSM."""
    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_RELEASE, 'Release: 7.9')

    release = rhsm.get_release(context_mocked)

    assert release, 'No release information detected (but valid release info was provided).'
    assert release == '7.9', 'Detected release is incorrect.'


def test_get_release_with_release_not_set(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library does not retrieve release information when the release is not set."""
    # Test whether no release is detected correctly too
    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_RELEASE, 'Release not set')

    release = rhsm.get_release(context_mocked)

    fail_description = 'The release information was obtained, even if "No release set" was reported by rhsm.'
    assert not release, fail_description


def test_is_manifest_sca_on_nonsca_system(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library obtains the SCA information correctly from a non-SCA system."""
    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_STATUS, RHSM_STATUS_OUTPUT_NOSCA)

    is_sca = rhsm.is_manifest_sca(context_mocked)
    assert not is_sca, 'SCA was detected on a non-SCA system.'


def test_is_manifest_sca_on_sca_system(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library obtains the SCA information from SCA system correctly."""
    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_STATUS, RHSM_STATUS_OUTPUT_SCA)

    is_sca = rhsm.is_manifest_sca(context_mocked)
    assert is_sca, 'Failed to detected SCA on a SCA system.'


def test_get_enabled_repo_ids(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library retrieves correct information about enabled repositories."""
    # Prepare the (realistic) RHSM output
    rhsm_list_enabled_output = '''
    +----------------------------------------------------------+
       Available Repositories in /etc/yum.repos.d/redhat.repo
    +----------------------------------------------------------+
    '''

    for enabled_repository in RHSM_ENABLED_REPOS:
        rhsm_output_fragment = 'Repo ID: {0}\n'.format(enabled_repository.id)
        rhsm_output_fragment += 'Repo Name: {0}\n'.format(enabled_repository.name)
        rhsm_output_fragment += 'Repo URL: {0}\n'.format(enabled_repository.url)
        rhsm_output_fragment += 'Enabled: {0}\n'.format(enabled_repository.enabled)
        rhsm_output_fragment += '\n'
        rhsm_list_enabled_output += rhsm_output_fragment

    context_mocked.add_mocked_command_call_with_stdout(CMD_RHSM_LIST_ENABLED_REPOS, rhsm_list_enabled_output)

    enabled_repo_ids = rhsm.get_enabled_repo_ids(context_mocked)

    fail_description = 'Failed to detected enabled repositories on the system.'
    assert len(enabled_repo_ids) == 3, fail_description

    fail_description = 'Failed to retrieve repository ID provided in the RHSM output.'
    for enabled_repository in RHSM_ENABLED_REPOS:
        assert enabled_repository.id in enabled_repo_ids, fail_description


def test_get_existing_product_certificates(monkeypatch, actor_mocked, context_mocked):
    """Verifies that the library is able to correctly retrieve existing product certificates."""

    CERT_DIRS_LAYOUT = {
        '/etc/pki/product': ['cert1', 'cert2'],
        '/etc/pki/product-default': ['cert3']
    }

    def mocked_isdir(path):
        if path in CERT_DIRS_LAYOUT:
            return True
        err_message = 'RHSM library should not gather info about additional dirs (attempted to isdir: {0}).'
        raise ValueError(err_message.format(path))

    def mocked_listdir(path):
        if path in CERT_DIRS_LAYOUT:
            return CERT_DIRS_LAYOUT[path]
        err_message = 'RHSM library should not listdir additional dirs (attempted to listdir: {0}).'
        raise ValueError(err_message.format(path))

    def mocked_isfile(path):
        if path in CERT_DIRS_LAYOUT:
            # The certificate directories are not files
            return False

        basename = os.path.basename(path)
        dirname = os.path.dirname(path)
        if dirname in CERT_DIRS_LAYOUT:
            return basename in CERT_DIRS_LAYOUT[dirname]

        err_message = 'RHSM library should not isfile additional paths (attempted to isfile: {0}).'
        raise ValueError(err_message.format(path))

    monkeypatch.setattr(rhsm.os.path, 'isdir', mocked_isdir)
    monkeypatch.setattr(rhsm.os, 'listdir', mocked_listdir)
    monkeypatch.setattr(rhsm.os.path, 'isfile', mocked_isfile)

    existing_product_certificates = rhsm.get_existing_product_certificates(context_mocked)

    fail_description = 'Retrieved different number of certificates than expected.'
    assert len(existing_product_certificates) == 3, fail_description

    fail_description_bad_dir = 'Found certificate in unexpected path: {0}'
    fail_description_bad_cert_file = 'Found certificate file that was not provided by mocked output: {0}'
    for certificate_path in existing_product_certificates:
        dirname = os.path.dirname(certificate_path)
        basename = os.path.basename(certificate_path)
        assert dirname in CERT_DIRS_LAYOUT, fail_description_bad_dir.format(certificate_path)
        assert basename in CERT_DIRS_LAYOUT[dirname], fail_description_bad_cert_file.format(certificate_path)


def test_get_existing_product_certificates_missing_cert_directory(monkeypatch, actor_mocked, context_mocked):
    """Tests whether the library is able to retrieve certificates even if /etc/pki/product is missing."""

    def mocked_isdir(path):
        if path == '/etc/pki/product':
            return False  # Directory is missing
        if path == '/etc/pki/product-default':
            return True

        err_msg = 'Tried to isdir a path that is not a part of the mocked paths. Path: {0}'
        raise ValueError(err_msg.format(path))

    def mocked_isfile(path):
        if path == '/etc/pki/product-default/cert':
            return True

        err_msg = 'Tried to use isfile on a path that is not a part of the mocked paths. Path: {0}'
        raise ValueError(err_msg.format(path))

    def mocked_listdir(path):
        if path == '/etc/pki/product-default':
            return ['cert']

        err_msg = 'Tried to use listdir on a path that is not a part of the mocked paths. Path: {0}'
        raise ValueError(err_msg.format(path))

    monkeypatch.setattr(rhsm.os.path, 'isdir', mocked_isdir)
    monkeypatch.setattr(rhsm.os, 'listdir', mocked_listdir)
    monkeypatch.setattr(rhsm.os.path, 'isfile', mocked_isfile)

    existing_product_certificates = rhsm.get_existing_product_certificates(context_mocked)

    fail_description = 'Library identified more certificates than there are in mocked outputs.'
    assert len(existing_product_certificates) == 1, fail_description
    fail_description = 'Library failed to identify certificate from mocked outputs.'
    assert existing_product_certificates[0] == '/etc/pki/product-default/cert', fail_description



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.stdlib import config
from leapp.models import Report, SkippedRepositories
from leapp.reporting import create_report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckSkippedRepositories(Actor):
    """
    Produces a report if any repositories enabled on the system are going to be skipped.

    The report produced by this actor should additionally include any package that is affected due to skipping
    the repository.
    """

    name = 'check_skipped_repositories'
    consumes = (SkippedRepositories,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        repos = set()
        packages = set()

        for message in self.consume(SkippedRepositories):
            repos.update(message.repos)
            packages.update(message.packages)

        if repos:
            title = 'Some enabled RPM repositories are unknown to Leapp'
            summary_data = []
            summary_data.append('The following repositories with Red Hat-signed packages are unknown to Leapp:')
            summary_data.extend(['- {}'.format(r) for r in repos])
            summary_data.append('And the following packages installed from those repositories may not be upgraded:')
            summary_data.extend(['- {}'.format(p) for p in packages])
            summary = '\n'.join(summary_data)

            packages_related = [reporting.RelatedResource('package', str(p)) for p in packages]
            repos_related = [reporting.RelatedResource('repository', str(r)) for r in repos]

            create_report([
                reporting.Title(title),
                reporting.Summary(summary),
                reporting.Severity(reporting.Severity.LOW),
                reporting.Groups([reporting.Groups.REPOSITORY]),
                reporting.Remediation(
                    hint='You can file a request to add this repository to the scope of in-place upgrades '
                         'by filing a support ticket')
            ] + packages_related + repos_related)

            if config.is_verbose():
                self.log.info('\n'.join([title, summary]))



from leapp.actors import Actor
from leapp.libraries.common import dnfplugin
from leapp.models import (
    DNFPluginTask,
    DNFWorkaround,
    FilteredRpmTransactionTasks,
    StorageInfo,
    TargetOSInstallationImage,
    TargetUserSpaceInfo,
    UsedTargetRepositories,
    XFSPresence
)
from leapp.tags import IPUWorkflowTag, TargetTransactionChecksPhaseTag


class DnfTransactionCheck(Actor):
    """
    This actor tries to solve the RPM transaction to verify the all package dependencies can be successfully resolved.
    """

    name = 'dnf_transaction_check'
    consumes = (
        DNFPluginTask,
        DNFWorkaround,
        FilteredRpmTransactionTasks,
        StorageInfo,
        TargetOSInstallationImage,
        TargetUserSpaceInfo,
        UsedTargetRepositories,
        XFSPresence,
    )
    produces = ()
    tags = (IPUWorkflowTag, TargetTransactionChecksPhaseTag)

    def process(self):
        xfs_info = next(self.consume(XFSPresence), XFSPresence())
        storage_info = next(self.consume(StorageInfo), StorageInfo())
        used_repos = self.consume(UsedTargetRepositories)
        plugin_info = list(self.consume(DNFPluginTask))
        tasks = next(self.consume(FilteredRpmTransactionTasks), FilteredRpmTransactionTasks())
        target_userspace_info = next(self.consume(TargetUserSpaceInfo), None)
        target_iso = next(self.consume(TargetOSInstallationImage), None)

        if target_userspace_info:
            dnfplugin.perform_transaction_check(
                tasks=tasks, used_repos=used_repos, target_userspace_info=target_userspace_info,
                xfs_info=xfs_info, storage_info=storage_info, plugin_info=plugin_info, target_iso=target_iso
            )



from leapp.actors import Actor
from leapp.libraries.actor import rpmscanner
from leapp.models import InstalledRPM
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class RpmScanner(Actor):
    """
    Provides data about installed RPM Packages.

    After collecting data from RPM query, a message with relevant data will be produced.
    """

    name = 'rpm_scanner'
    consumes = ()
    produces = (InstalledRPM,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        rpmscanner.process()



import os

from leapp.libraries.actor import xfsinfoscanner
from leapp.libraries.common.testutils import produce_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import FstabEntry, MountEntry, StorageInfo, SystemdMountEntry, XFSPresence


class run_mocked(object):
    def __init__(self):
        self.called = 0
        self.args = None

    def __call__(self, args, split=True):
        self.called += 1
        self.args = args

        with_ftype = {'stdout': [
            "meta-data=/dev/loop0             isize=512    agcount=4, agsize=131072 blks",
            "         =                       sectsz=512   attr=2, projid32bit=1",
            "         =                       crc=1        finobt=0 spinodes=0",
            "data     =                       bsize=4096   blocks=524288, imaxpct=25",
            "         =                       sunit=0      swidth=0 blks",
            "naming   =version 2              bsize=4096   ascii-ci=0 ftype=1",
            "log      =internal               bsize=4096   blocks=2560, version=2",
            "         =                       sectsz=512   sunit=0 blks, lazy-count=1",
            "realtime =none                   extsz=4096   blocks=0, rtextents=0"]}

        without_ftype = {'stdout': [
            "meta-data=/dev/loop0             isize=512    agcount=4, agsize=131072 blks",
            "         =                       sectsz=512   attr=2, projid32bit=1",
            "         =                       crc=1        finobt=0 spinodes=0",
            "data     =                       bsize=4096   blocks=524288, imaxpct=25",
            "         =                       sunit=0      swidth=0 blks",
            "naming   =version 2              bsize=4096   ascii-ci=0 ftype=0",
            "log      =internal               bsize=4096   blocks=2560, version=2",
            "         =                       sectsz=512   sunit=0 blks, lazy-count=1",
            "realtime =none                   extsz=4096   blocks=0, rtextents=0"]}

        if "/var" in self.args:
            return without_ftype

        return with_ftype


def test_scan_xfs_fstab(monkeypatch):
    fstab_data_no_xfs = {
        "fs_spec": "/dev/mapper/fedora-home",
        "fs_file": "/home",
        "fs_vfstype": "ext4",
        "fs_mntops": "defaults,x-systemd.device-timeout=0",
        "fs_freq": "1",
        "fs_passno": "2"}

    mountpoints = xfsinfoscanner.scan_xfs_fstab([FstabEntry(**fstab_data_no_xfs)])
    assert not mountpoints

    fstab_data_xfs = {
        "fs_spec": "/dev/mapper/rhel-root",
        "fs_file": "/",
        "fs_vfstype": "xfs",
        "fs_mntops": "defaults",
        "fs_freq": "0",
        "fs_passno": "0"}

    mountpoints = xfsinfoscanner.scan_xfs_fstab([FstabEntry(**fstab_data_xfs)])
    assert mountpoints == {"/"}


def test_scan_xfs_mount(monkeypatch):
    mount_data_no_xfs = {
        "name": "tmpfs",
        "mount": "/run/snapd/ns",
        "tp": "tmpfs",
        "options": "rw,nosuid,nodev,seclabel,mode=755"}

    mountpoints = xfsinfoscanner.scan_xfs_mount([MountEntry(**mount_data_no_xfs)])
    assert not mountpoints

    mount_data_xfs = {
        "name": "/dev/vda1",
        "mount": "/boot",
        "tp": "xfs",
        "options": "rw,relatime,seclabel,attr2,inode64,noquota"}

    mountpoints = xfsinfoscanner.scan_xfs_mount([MountEntry(**mount_data_xfs)])
    assert mountpoints == {"/boot"}


def test_is_xfs_without_ftype(monkeypatch):
    monkeypatch.setattr(xfsinfoscanner, "run", run_mocked())
    monkeypatch.setattr(os.path, "ismount", lambda _: True)

    assert xfsinfoscanner.is_xfs_without_ftype("/var")
    assert ' '.join(xfsinfoscanner.run.args) == "/usr/sbin/xfs_info /var"

    assert not xfsinfoscanner.is_xfs_without_ftype("/boot")
    assert ' '.join(xfsinfoscanner.run.args) == "/usr/sbin/xfs_info /boot"


def test_is_xfs_command_failed(monkeypatch):
    def _run_mocked_exception(*args, **kwargs):
        raise CalledProcessError(message="No such file or directory", command=["xfs_info", "/nosuchmountpoint"],
                                 result=1)
    # not a mountpoint
    monkeypatch.setattr(os.path, "ismount", lambda _: False)
    monkeypatch.setattr(xfsinfoscanner, "run", _run_mocked_exception)
    assert not xfsinfoscanner.is_xfs_without_ftype("/nosuchmountpoint")
    # a real mountpoint but something else caused command to fail
    monkeypatch.setattr(os.path, "ismount", lambda _: True)
    assert not xfsinfoscanner.is_xfs_without_ftype("/nosuchmountpoint")


def test_scan_xfs(monkeypatch):
    monkeypatch.setattr(xfsinfoscanner, "run", run_mocked())
    monkeypatch.setattr(os.path, "ismount", lambda _: True)

    def consume_no_xfs_message_mocked(*models):
        yield StorageInfo()

    monkeypatch.setattr(api, "consume", consume_no_xfs_message_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    xfsinfoscanner.scan_xfs()
    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    assert isinstance(api.produce.model_instances[0], XFSPresence)
    assert not api.produce.model_instances[0].present
    assert not api.produce.model_instances[0].without_ftype
    assert not api.produce.model_instances[0].mountpoints_without_ftype

    def consume_ignored_xfs_message_mocked(*models):
        mount_data = {
            "name": "/dev/vda1",
            "mount": "/boot",
            "tp": "xfs",
            "options": "rw,relatime,seclabel,attr2,inode64,noquota"}
        yield StorageInfo(mount=[MountEntry(**mount_data)])

    monkeypatch.setattr(api, "consume", consume_ignored_xfs_message_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    xfsinfoscanner.scan_xfs()
    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    assert isinstance(api.produce.model_instances[0], XFSPresence)
    assert api.produce.model_instances[0].present
    assert not api.produce.model_instances[0].without_ftype
    assert not api.produce.model_instances[0].mountpoints_without_ftype

    def consume_xfs_with_ftype_message_mocked(*models):
        fstab_data = {
            "fs_spec": "/dev/mapper/rhel-root",
            "fs_file": "/",
            "fs_vfstype": "xfs",
            "fs_mntops": "defaults",
            "fs_freq": "0",
            "fs_passno": "0"}
        yield StorageInfo(fstab=[FstabEntry(**fstab_data)])

    monkeypatch.setattr(api, "consume", consume_xfs_with_ftype_message_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    xfsinfoscanner.scan_xfs()
    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    assert isinstance(api.produce.model_instances[0], XFSPresence)
    assert api.produce.model_instances[0].present
    assert not api.produce.model_instances[0].without_ftype
    assert not api.produce.model_instances[0].mountpoints_without_ftype

    def consume_xfs_without_ftype_message_mocked(*models):
        fstab_data = {
            "fs_spec": "/dev/mapper/rhel-root",
            "fs_file": "/var",
            "fs_vfstype": "xfs",
            "fs_mntops": "defaults",
            "fs_freq": "0",
            "fs_passno": "0"}
        yield StorageInfo(fstab=[FstabEntry(**fstab_data)])

    monkeypatch.setattr(api, "consume", consume_xfs_without_ftype_message_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    xfsinfoscanner.scan_xfs()
    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    assert isinstance(api.produce.model_instances[0], XFSPresence)
    assert api.produce.model_instances[0].present
    assert api.produce.model_instances[0].without_ftype
    assert api.produce.model_instances[0].mountpoints_without_ftype
    assert len(api.produce.model_instances[0].mountpoints_without_ftype) == 1
    assert api.produce.model_instances[0].mountpoints_without_ftype[0] == '/var'

    def consume_no_message_mocked(*models):
        yield None

    monkeypatch.setattr(api, "consume", consume_no_message_mocked)
    monkeypatch.setattr(api, "produce", produce_mocked())

    xfsinfoscanner.scan_xfs()
    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    assert isinstance(api.produce.model_instances[0], XFSPresence)
    assert not api.produce.model_instances[0].present
    assert not api.produce.model_instances[0].without_ftype
    assert not api.produce.model_instances[0].mountpoints_without_ftype



import os

from leapp import reporting

CUSTOM_NETWORK_SCRIPTS = [
    "/sbin/ifup-local",
    "/sbin/ifup-pre-local",
    "/sbin/ifdown-local",
    "/sbin/ifdown-pre-local",
]
DOC_URL = "https://red.ht/upgrading-RHEL-8-to-RHEL-9-network-scripts"


def generate_report(existing_custom_network_scripts):
    """ Generate reports informing user about possible manual intervention required """

    # Show documentation url if custom network-scripts detected
    title = "custom network-scripts detected"
    summary = (
        "RHEL 9 does not support the legacy network-scripts package that was"
        " deprecated in RHEL 8. Custom network-scripts have been detected."
    )

    reporting.create_report(
        [
            reporting.Title(title),
            reporting.Summary(summary),
            reporting.Remediation(
                hint=(
                    "Migrate the custom network-scripts to NetworkManager dispatcher"
                    " scripts manually before the ugprade. Follow instructions in the"
                    " official documentation."
                )
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.NETWORK, reporting.Groups.SERVICES]),
            reporting.ExternalLink(
                title=(
                    "Upgrading from RHEL 8 to 9 - migrating custom network-scripts to"
                    " NetworkManager dispatcher scripts"
                ),
                url=DOC_URL,
            ),
        ]
        + [
            reporting.RelatedResource("file", fname)
            for fname in existing_custom_network_scripts
        ]
    )


def process():
    existing_custom_network_scripts = [
        fname for fname in CUSTOM_NETWORK_SCRIPTS if os.path.isfile(fname)
    ]
    if existing_custom_network_scripts:
        generate_report(existing_custom_network_scripts)



import os

from leapp.libraries.actor import migrateblacklistca
from leapp.libraries.common.testutils import CurrentActorMocked
from leapp.libraries.stdlib import api


class MockedGetFiles():
    def __init__(self):
        self.called = 0

    def __call__(self):
        self.called += 1
        return []


def test_no_dirs_exist(monkeypatch):
    mocked_files = MockedGetFiles()
    monkeypatch.setattr(os.path, 'exists', lambda dummy: False)
    monkeypatch.setattr(migrateblacklistca, '_get_files', mocked_files)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    # this is bad mock, but we want to be sure that update-ca-trust is not
    # called on the testing machine
    monkeypatch.setattr(migrateblacklistca, 'run', lambda dummy: dummy)
    assert not mocked_files.called



from leapp.actors import Actor
from leapp.libraries.actor.removeoldpammodulesscanner import RemoveOldPAMModulesScannerLibrary
from leapp.libraries.common.pam import PAM
from leapp.models import RemovedPAMModules
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class RemoveOldPAMModulesScanner(Actor):
    """
    Scan PAM configuration for modules that are not available in RHEL-8.

    PAM module pam_krb5 and pam_pkcs11 are no longer present in RHEL-8
    and must be removed from PAM configuration, otherwise it may lock out
    the system.
    """
    name = 'removed_pam_modules_scanner'
    consumes = ()
    produces = (RemovedPAMModules,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        pam = PAM.from_system_configuration()
        scanner = RemoveOldPAMModulesScannerLibrary(pam)
        self.produce(scanner.process())



import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import mounting
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import CryptoPolicyInfo, TargetUserSpaceInfo

UCP_SCRIPT_PATH = '/usr/bin/update-crypto-policies'


def _set_crypto_policy(context, current_policy):
    if not os.path.exists(context.full_path(UCP_SCRIPT_PATH)):
        # consider this as heavy bug in actors and user cannot do anything - just report the bug
        raise StopActorExecutionError(
            'The update-crypto-policies tool is not installed inside the {} container.'
            .format(context.full_path('/'))
        )
    try:
        # In case of custom policies, it will most likely crash
        # TODO(pstodulk): copy custom files with the crypto policies to the container first
        context.call(['update-crypto-policies', '--set', current_policy])
    except CalledProcessError as e:
        # maybe in this case we could set an inhibitor instead?
        details = {'details:': str(e), 'stderr': e.stderr}

        # NOTE(pstodulk): In case the LEGACY crypto policy is set, I expect
        # a different root cause..
        if current_policy != 'LEGACY':
            details['hint'] = (
                'Custom crypto policies are likely used on the system.'
                ' The in-place upgrade cannot handle this correctly right now.'
                ' Consider the use of crypto policies that exists on the target system'
                ' or contact the support to prioritize the work on this feature.'
            )
        raise StopActorExecutionError(
            message='Cannot set the expected crypto policies: {}'.format(current_policy),
            details=details
        )


def process():
    target_userspace_info = next(api.consume(TargetUserSpaceInfo), None)
    if not target_userspace_info:
        # nothing to do - an error occurred in previous actors and upgrade will be inhibited
        api.current_logger().error('Missing the TargetUserSpaceInfo message. Probably it has not been created before.')
        return
    cpi = next(api.consume(CryptoPolicyInfo), None)
    if not cpi:
        # unexpected, but still better to log the error
        api.current_logger().error('Missing the CryptoPolicyInfo message. Nothing to do')
        return
    if cpi.current_policy == 'DEFAULT':
        api.current_logger().debug('The default crypto policy detected on the host system. Nothing to do.')
        return
    with mounting.NspawnActions(base_dir=target_userspace_info.path) as context:
        _set_crypto_policy(context, cpi.current_policy)



import pytest

from leapp.models import FIPSInfo, KernelCmdline, KernelCmdlineArg
from leapp.snactor.fixture import current_actor_context

ballast1 = [KernelCmdlineArg(key=k, value=v) for k, v in [
    ('BOOT_IMAGE', '/vmlinuz-3.10.0-1127.el7.x86_64'),
    ('root', '/dev/mapper/rhel_ibm--p8--kvm--03--guest--02-root'),
    ('ro', ''),
    ('console', 'tty0'),
    ('console', 'ttyS0,115200'),
    ('rd_NO_PLYMOUTH', '')]]
ballast2 = [KernelCmdlineArg(key=k, value=v) for k, v in [
    ('crashkernel', 'auto'),
    ('rd.lvm.lv', 'rhel_ibm-p8-kvm-03-guest-02/root'),
    ('rd.lvm.lv', 'rhel_ibm-p8-kvm-03-guest-02/swap'),
    ('rhgb', ''),
    ('quiet', ''),
    ('LANG', 'en_US.UTF-8')]]


@pytest.mark.parametrize(('parameters', 'should_detect_enabled_fips'), [
    ([], False),
    ([KernelCmdlineArg(key='fips', value='')], False),
    ([KernelCmdlineArg(key='fips', value='0')], False),
    ([KernelCmdlineArg(key='fips', value='1')], True),
    ([KernelCmdlineArg(key='fips', value='11')], False),
    ([KernelCmdlineArg(key='fips', value='yes')], False)
])
def test_check_fips(current_actor_context, parameters, should_detect_enabled_fips):
    cmdline = KernelCmdline(parameters=ballast1+parameters+ballast2)
    current_actor_context.feed(cmdline)
    current_actor_context.run()
    produced_msgs = current_actor_context.consume(FIPSInfo)

    assert (FIPSInfo(is_enabled=should_detect_enabled_fips),) == produced_msgs



from leapp.actors import Actor
from leapp.libraries.actor.opensshdeprecateddirectivescheck import inhibit_if_deprecated_directives_used
from leapp.models import OpenSshConfig, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class OpenSshDeprecatedDirectivesCheck(Actor):
    """
    Check for any deprecated directives in the OpenSSH configuration.

    Checks the directives used in the OpenSSH configuration for ones that have
    been deprecated and their usage in newer versions would result in the sshd
    service failing to start after the upgrade.
    """

    name = 'open_ssh_deprecated_directives_check'
    consumes = (OpenSshConfig,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        ssh_config = next(self.consume(OpenSshConfig))
        inhibit_if_deprecated_directives_used(ssh_config)



from leapp.actors import Actor
from leapp.libraries.common import dnfplugin
from leapp.models import (
    DNFPluginTask,
    DNFWorkaround,
    FilteredRpmTransactionTasks,
    RHUIInfo,
    StorageInfo,
    TargetOSInstallationImage,
    TargetUserSpaceInfo,
    UsedTargetRepositories,
    XFSPresence
)
from leapp.tags import DownloadPhaseTag, IPUWorkflowTag


class DnfPackageDownload(Actor):
    """
    Actor that invokes DNF to download the RPMs required for the upgrade transaction.

    This actor uses the rhel-upgrade plugin to perform the download of RPM for the transaction and performing the
    transaction test, that is something like a dry run trying to determine the success of the upgrade.
    """

    name = 'dnf_package_download'
    consumes = (
        DNFPluginTask,
        DNFWorkaround,
        FilteredRpmTransactionTasks,
        RHUIInfo,
        StorageInfo,
        TargetOSInstallationImage,
        TargetUserSpaceInfo,
        UsedTargetRepositories,
        XFSPresence,
    )
    produces = ()
    tags = (IPUWorkflowTag, DownloadPhaseTag)

    def process(self):
        xfs_info = next(self.consume(XFSPresence), XFSPresence())
        storage_info = next(self.consume(StorageInfo), StorageInfo())
        used_repos = self.consume(UsedTargetRepositories)
        plugin_info = list(self.consume(DNFPluginTask))
        tasks = next(self.consume(FilteredRpmTransactionTasks), FilteredRpmTransactionTasks())
        target_userspace_info = next(self.consume(TargetUserSpaceInfo), None)
        rhui_info = next(self.consume(RHUIInfo), None)
        # there are several "variants" related to the *AWS* provider (aws, aws-sap)
        on_aws = bool(rhui_info and rhui_info.provider.startswith('aws'))
        target_iso = next(self.consume(TargetOSInstallationImage), None)

        dnfplugin.perform_rpm_download(
            tasks=tasks, used_repos=used_repos, target_userspace_info=target_userspace_info,
            xfs_info=xfs_info, storage_info=storage_info, plugin_info=plugin_info, on_aws=on_aws,
            target_iso=target_iso
        )



def isIpsetTypeSupportedByNftables(ipset_type):
    if ipset_type in ['hash:ip', 'hash:mac', 'hash:net']:
        return True

    return False


def isEbtablesTableSupported(table):
    if table in ['filter', 'nat']:
        return True

    return False



import errno
import os

from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.stdlib import api, run
from leapp.reporting import create_report, Report
from leapp.tags import FirstBootPhaseTag, IPUWorkflowTag


class RemoveSystemdResumeService(Actor):
    """
    Remove systemd service to launch Leapp.

    After system was rebooted and process resumed, this service is not necessary anymore.
    """

    name = 'remove_systemd_resume_service'
    consumes = ()
    produces = (Report,)
    tags = (FirstBootPhaseTag.After, IPUWorkflowTag)

    def process(self):
        service_name = 'leapp_resume.service'
        if os.path.isfile('/etc/systemd/system/{}'.format(service_name)):
            run(['systemctl', 'disable', service_name])
            paths_to_unlink = [
                '/etc/systemd/system/{}'.format(service_name),
                '/etc/systemd/system/default.target.wants/{}'.format(service_name),
            ]
            for path in paths_to_unlink:
                try:
                    os.unlink(path)
                except OSError as e:
                    api.current_logger().debug('Failed removing {}: {}'.format(path, str(e)))
                    if e.errno != errno.ENOENT:
                        raise

        create_report([
            reporting.Title('"{}" service deleted'.format(service_name)),
            reporting.Summary(
                '"{}" was taking care of resuming upgrade process '
                'after the first reboot.'.format(service_name)),
            reporting.Groups([reporting.Groups.UPGRADE_PROCESS]),
        ])



import pytest

from leapp import reporting
from leapp.libraries.actor import checkselinux
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api
from leapp.models import Report, SELinuxFacts, SelinuxPermissiveDecision, SelinuxRelabelDecision
from leapp.snactor.fixture import current_actor_context


def create_selinuxfacts(static_mode, enabled, policy='targeted', mls_enabled=True):
    runtime_mode = static_mode if static_mode != 'disabled' else None

    return SELinuxFacts(
        runtime_mode=runtime_mode,
        static_mode=static_mode,
        enabled=enabled,
        policy=policy,
        mls_enabled=mls_enabled
    )


@pytest.mark.parametrize('mode', ('permissive', 'enforcing'))
def test_actor_schedule_relabelling(monkeypatch, mode):

    fact = create_selinuxfacts(static_mode=mode, enabled=True)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[fact]))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())

    checkselinux.process()

    assert api.produce.model_instances[0].set_relabel
    assert reporting.create_report.called


def test_actor_set_permissive(monkeypatch):
    relabel = create_selinuxfacts(static_mode='enforcing', enabled=True)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[relabel]))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())

    checkselinux.process()

    assert api.produce.model_instances[0].set_relabel
    assert api.produce.model_instances[1].set_permissive
    assert reporting.create_report.called


@pytest.mark.parametrize('el8_to_el9', (True, False))
def test_actor_selinux_disabled(monkeypatch, el8_to_el9):
    disabled = create_selinuxfacts(enabled=False, static_mode='disabled')

    target_ver = '8' if not el8_to_el9 else '9'

    monkeypatch.setattr(checkselinux, 'get_target_major_version', lambda: target_ver)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[disabled]))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())

    checkselinux.process()
    if el8_to_el9:
        assert api.produce.model_instances[0]
        assert reporting.create_report.called == 2
    else:
        assert not api.produce.model_instances
        assert reporting.create_report.called



import errno
import re

from leapp.libraries.common.spamassassinutils import SPAMC_CONFIG_FILE
from leapp.libraries.stdlib import api


def _remove_comments(content):
    return re.sub(r'^#.*$', '', content, flags=re.MULTILINE)


def _parse_spamc_ssl_argument(content):
    content = _remove_comments(content)
    res = None
    for match in re.finditer(r'(?<!\S)--ssl(\s+|=)(sslv3|tlsv1)(?!\S)', content):
        arg = match.group(2)
        if arg == 'tlsv1' or (arg == 'sslv3' and res is None):
            res = arg
    return res


def get_spamc_ssl_argument(read_func):
    """
    Extracts and returns the argument given to the --ssl option from the spamc
    configuration file. Returns None if the option is not specified or the config
    file does not exist.
    """
    try:
        content = read_func(SPAMC_CONFIG_FILE)
        return _parse_spamc_ssl_argument(content)
    except (IOError, OSError) as e:
        if e.errno != errno.ENOENT:
            api.current_logger().warning(
                'Failed to read spamc configuration file: %s' % e)
        return None



from leapp.actors import Actor
from leapp.libraries.actor import distributionsignedrpmscanner
from leapp.models import DistributionSignedRPM, InstalledRedHatSignedRPM, InstalledRPM, InstalledUnsignedRPM
from leapp.tags import FactsPhaseTag, IPUWorkflowTag
from leapp.utils.deprecation import suppress_deprecation


@suppress_deprecation(InstalledRedHatSignedRPM)
class DistributionSignedRpmScanner(Actor):
    """
    Provide data about distribution signed & unsigned RPM packages.

    For various checks and actions done during the upgrade it's important to
    know what packages are signed by GPG keys of the installed linux system
    distribution. RPMs that are not provided in the distribution could have
    different versions, different behaviour, and also it could be completely
    different application just with the same RPM name.

    For that reasons, various actors rely on the DistributionSignedRPM message
    to check whether particular package is installed, to be sure it provides
    valid data. Fingerprints of distribution GPG keys are stored under
      common/files/distro/<distro>/gpg_signatures.json
    where <distro> is distribution ID of the installed system (e.g. centos, rhel).

    If the file for the installed distribution is not find, end with error.
    """

    name = 'distribution_signed_rpm_scanner'
    consumes = (InstalledRPM,)
    produces = (DistributionSignedRPM, InstalledRedHatSignedRPM, InstalledUnsignedRPM,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        distributionsignedrpmscanner.process()



import sys

import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor import enablerhsmtargetrepos
from leapp.libraries.common import config, mounting, rhsm
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import EnvVar, UsedTargetRepositories, UsedTargetRepository


def not_isolated_actions(raise_err=False):
    commands_called = []

    class MockNotIsolatedActions(object):
        def __init__(self, base_dir=None):
            pass

        def call(self, cmd, **kwargs):
            commands_called.append((cmd, kwargs))
            if raise_err:
                raise_call_error()

    return (commands_called, MockNotIsolatedActions)


def raise_call_error(args=None):
    raise CalledProcessError(
        message='A Leapp Command Error occurred.',
        command=args,
        result={'signal': None, 'exit_code': 1, 'pid': 0, 'stdout': 'fake', 'stderr': 'fake'}
    )


class run_mocked(object):
    def __init__(self, raise_err=False):
        self.called = 0
        self.args = []
        self.raise_err = raise_err

    def __call__(self, *args):
        self.called += 1
        self.args.append(args)
        if self.raise_err:
            raise_call_error(args)


def test_setrelease(monkeypatch):
    commands_called, klass = not_isolated_actions()
    monkeypatch.setattr(mounting, 'NotIsolatedActions', klass)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='8.0'))
    monkeypatch.setattr(config, 'get_product_type', lambda dummy: 'ga')
    enablerhsmtargetrepos.set_rhsm_release()
    assert commands_called and len(commands_called) == 1
    assert commands_called[0][0][-1] == '8.0'


def test_setrelease_submgr_throwing_error(monkeypatch):
    _, klass = not_isolated_actions(raise_err=True)
    monkeypatch.setattr(mounting, 'NotIsolatedActions', klass)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='8.0', envars={'LEAPP_NO_RHSM': '0'}))
    monkeypatch.setattr(config, 'get_product_type', lambda dummy: 'ga')
    # free the set_release function from the @_rhsm_retry decorator
    # which would otherwise cause 25 sec delay of the test
    if sys.version_info.major < 3:
        monkeypatch.setattr(rhsm, 'set_release',
                            rhsm.set_release.func_closure[0].cell_contents.func_closure[0].cell_contents)
    else:
        monkeypatch.setattr(rhsm, 'set_release', rhsm.set_release.__wrapped__.__wrapped__)
    with pytest.raises(StopActorExecutionError):
        enablerhsmtargetrepos.set_rhsm_release()


@pytest.mark.parametrize('product', ['beta', 'htb'])
def test_setrelease_skip_rhsm(monkeypatch, product):
    commands_called, _ = not_isolated_actions()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars={'LEAPP_NO_RHSM': '1'}))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(config, 'get_product_type', lambda dummy: product)
    # To make this work we need to re-apply the decorator, so it respects the environment variable
    monkeypatch.setattr(rhsm, 'set_release', rhsm.with_rhsm(rhsm.set_release))
    enablerhsmtargetrepos.set_rhsm_release()
    assert not commands_called


def construct_UTRepo_consume(repoids):
    repos = [UsedTargetRepository(repoid=repoid) for repoid in repoids]
    return lambda *x: (x for x in (UsedTargetRepositories(repos=repos),))


def test_get_unique_repoids(monkeypatch):
    repoids = (['some-repo', 'some-repo', 'another-repo'])
    monkeypatch.setattr(api, 'consume', construct_UTRepo_consume(repoids))
    assert enablerhsmtargetrepos.get_repos_to_enable() == {'some-repo', 'another-repo'}


def test_get_submgr_cmd():
    assert enablerhsmtargetrepos.get_submgr_cmd({'some-repo'}) == ['subscription-manager', 'repos', '--enable',
                                                                   'some-repo']


def test_running_submgr_ok(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='8.0', envars={'LEAPP_NO_RHSM': '0'}), )
    monkeypatch.setattr(enablerhsmtargetrepos, 'get_repos_to_enable', lambda: {'some-repo'})
    monkeypatch.setattr(enablerhsmtargetrepos, 'run', run_mocked())
    enablerhsmtargetrepos.enable_rhsm_repos()
    assert enablerhsmtargetrepos.run.called
    assert 'subscription-manager' in enablerhsmtargetrepos.run.args[0][0]


def test_running_submgr_fail(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='8.0', envars={'LEAPP_NO_RHSM': '0'}), )
    monkeypatch.setattr(enablerhsmtargetrepos, 'get_repos_to_enable', lambda: {'some-repo'})
    monkeypatch.setattr(enablerhsmtargetrepos, 'run', run_mocked(raise_err=True))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    enablerhsmtargetrepos.enable_rhsm_repos()
    assert enablerhsmtargetrepos.run.called
    assert api.current_logger.warnmsg


def test_enable_repos_skip_rhsm(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars={'LEAPP_NO_RHSM': '1'}))
    monkeypatch.setattr(enablerhsmtargetrepos, 'run', run_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    enablerhsmtargetrepos.enable_rhsm_repos()
    assert not enablerhsmtargetrepos.run.called
    assert api.current_logger.dbgmsg



from leapp.actors import Actor
from leapp.libraries.actor.rpmtransactionconfigtaskscollector import load_tasks
from leapp.models import DistributionSignedRPM, RpmTransactionTasks
from leapp.tags import FactsPhaseTag, IPUWorkflowTag

CONFIGURATION_BASE_PATH = '/etc/leapp/transaction'


class RpmTransactionConfigTasksCollector(Actor):
    """
    Provides additional RPM transaction tasks from /etc/leapp/transaction.

    After collecting task data from /etc/leapp/transaction directory, a message with relevant data
    will be produced.
    """

    name = 'rpm_transaction_config_tasks_collector'
    consumes = (DistributionSignedRPM,)
    produces = (RpmTransactionTasks,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        self.produce(load_tasks(CONFIGURATION_BASE_PATH, self.log))



import textwrap

from leapp.libraries.common.pam import PAM


def get_config(config):
    return textwrap.dedent(config).strip()


def test_PAM_parse():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    ''')

    obj = PAM('')
    modules = obj.parse(pam)

    assert len(modules) == 3
    assert 'pam_unix' in modules
    assert 'pam_sss' in modules
    assert 'pam_deny' in modules


def test_PAM_has__true():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    ''')

    obj = PAM(pam)
    assert obj.has('pam_unix')
    assert obj.has('pam_sss')
    assert obj.has('pam_deny')


def test_PAM_has__false():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    ''')

    obj = PAM(pam)
    assert not obj.has('pam_winbind')


def test_PAM_has_unknown_module__empty():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    ''')

    obj = PAM(pam)
    assert obj.has_unknown_module([])


def test_PAM_has_unknown_module__false():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    ''')

    obj = PAM(pam)
    assert not obj.has_unknown_module(['pam_unix', 'pam_sss', 'pam_deny'])


def test_PAM_has_unknown_module__true():
    pam = get_config('''
    auth sufficient pam_unix.so
    auth sufficient pam_sss.so
    auth required pam_deny.so
    session pam_ecryptfs.so
    ''')

    obj = PAM(pam)
    assert obj.has_unknown_module(['pam_unix', 'pam_sss', 'pam_deny'])


def test_PAM_read_file__non_existent():
    content = PAM.read_file('/this/does/not/exist')
    assert content == ''


def test_PAM_read_file__ok():
    content = PAM.read_file(__file__)
    assert content != ''
    assert 'test_PAM_read_file__ok' in content



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class SSSDDomainConfig(Model):
    """
    Facts found about an SSSD domain.
    """
    topic = SystemInfoTopic

    name = fields.String()
    """
    Domain name.
    """

    options = fields.List(fields.String(), default=list())
    """
    List of options related to this domain that affects the upgrade process.
    """


class SSSDConfig(Model):
    """
    List of SSSD domains and their configuration that is related to the
    upgrade process.
    """
    topic = SystemInfoTopic

    domains = fields.List(fields.Model(SSSDDomainConfig), default=list())
    """
    SSSD Domains configuration.
    """



from leapp.actors import Actor
from leapp.libraries.actor.checkpersistentmounts import check_persistent_mounts
from leapp.models import StorageInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckPersistentMounts(Actor):
    """
    Check if mounts required to be persistent are mounted in persistent fashion.

    Checks performed:
        - if /var/lib/leapp is mounted it has an entry in /etc/fstab
    """
    name = "check_persistent_mounts"
    consumes = (StorageInfo,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        check_persistent_mounts()



import os

from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import BlackListCA, BlackListError

# dict(orig_dir: new_dir)
DIRS_CHANGE = {
    '/etc/pki/ca-trust/source/blacklist/': '/etc/pki/ca-trust/source/blocklist/',
    '/usr/share/pki/ca-trust-source/blacklist/': '/usr/share/pki/ca-trust-source/blocklist/'
}


def _get_dirs():
    return DIRS_CHANGE


def _get_files(dirname):
    """
    :raises: CalledProcessError: if the find command fails
    """
    # on rhel8, -type can't take two arguments, so we need to call find
    # twice and concatenate the results
    files = run(['find', dirname, '-type', 'f'], split=True)['stdout']
    return files + run(['find', dirname, '-type', 'l'], split=True)['stdout']


def _generate_messages(dirname, targetname):
    if not os.path.exists(dirname):
        # The directory does not exist; not an error (there is just nothing
        # to migrate).
        return
    try:
        blacklisted_certs = _get_files(dirname)
    except (CalledProcessError) as e:
        api.produce(BlackListError(sourceDir=dirname, targetDir=targetname, error=str(e)))
        api.current_logger().error('Cannot get list of files in {}: {}.'.format(dirname, e))
        return
    for filename in blacklisted_certs:
        # files found, pass a message to the reporter.
        # (maybe to migrateblacklistca as well)
        target = filename.replace(dirname, targetname)
        api.produce(BlackListCA(source=filename, sourceDir=dirname, target=target, targetDir=targetname))


def process():
    change_dirs = _get_dirs()
    for dirname in change_dirs:
        _generate_messages(dirname, change_dirs[dirname])



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class GrubConfigError(Model):
    ERROR_CORRUPTED_GRUBENV = 'corrupted grubenv'
    ERROR_MISSING_NEWLINE = 'missing newline'
    ERROR_GRUB_CMDLINE_LINUX_SYNTAX = 'GRUB_CMDLINE_LINUX syntax'

    topic = SystemFactsTopic

    # XXX FIXME(ivasilev) Rename to error_resolvable?
    # If error can be automatically resolved (ex. in addupgradebootentry actor)
    error_detected = fields.Boolean(default=False)
    error_type = fields.StringEnum([ERROR_CORRUPTED_GRUBENV, ERROR_MISSING_NEWLINE, ERROR_GRUB_CMDLINE_LINUX_SYNTAX])
    # Paths to config files
    files = fields.List(fields.String())



from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxModules


def remove_semanage_customizations():
    # remove SELinux customizations done by semanage -- to be reintroduced after the upgrade
    api.current_logger().info('Removing SELinux customizations introduced by semanage.')

    semanage_options = ['login', 'user', 'port', 'interface', 'module', 'node',
                        'fcontext', 'boolean', 'ibpkey', 'ibendport']
    # permissive domains are handled by porting modules (permissive -a adds new cil module with priority 400)
    for option in semanage_options:
        try:
            run(['semanage', option, '-D'])
        except CalledProcessError:
            continue


# remove custom SElinux modules - to be reinstalled after the upgrade
def remove_custom_modules():
    # go through all SELinuxModules messages -- in theory there should be only one
    for semodules in api.consume(SELinuxModules):
        api.current_logger().info('Removing custom SELinux policy modules. Count: {}'.format(len(semodules.modules)))

        # Form a single "semodule" command to remove all given modules at once.
        # This will help with any inter-dependencies.
        # "semodule" continues removing modules even if it encounters errors - any issues are just printed to stdout
        if semodules.modules:
            command = ['semodule']

            for module in semodules.modules:
                command.extend(['-X', str(module.priority), '-r', module.name])
            try:
                run(command)
            except CalledProcessError as e:
                api.current_logger().warning(
                    'Error removing modules in a single transaction:'
                    '{}\nRetrying -- now each module will be removed separately.'.format(e.stderr)
                )
                # Retry, but remove each module separately
                for module in semodules.modules:
                    try:
                        run(['semodule', '-X', str(module.priority), '-r', module.name])
                    except CalledProcessError as e:
                        api.current_logger().warning('Failed to remove module {} on priority {}: {}'.format(
                                                module.name, module.priority, e.stderr))
                        continue

        remove_udica_templates(semodules.templates)


def remove_udica_templates(templates):
    if not templates:
        return

    api.current_logger().info('Removing "udica" policy templates. Count: {}'.format(len(templates)))
    command = ['semodule']

    for module in templates:
        command.extend(['-X', str(module.priority), '-r', module.name])
    try:
        run(command)
    except CalledProcessError as e:
        api.current_logger().warning(
            'Failed to remove some "udica" policy templates: {}'.format(e.stderr)
        )



from leapp import reporting
from leapp.actors import Actor
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class PythonInformUser(Actor):
    name = "python_inform_user"
    description = "This actor informs the user of differences in Python version and support in RHEL 8."
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        url = "https://red.ht/rhel-8-python"
        title = "Difference in Python versions and support in RHEL 8"
        summary = ("In RHEL 8, there is no 'python' command."
                   " Python 3 (backward incompatible) is the primary Python version"
                   " and Python 2 is available with limited support and limited set of packages."
                   " If you no longer require Python 2 packages following the upgrade, please remove them."
                   " Read more here: {}".format(url))
        create_report([
            reporting.Title(title),
            reporting.Summary(summary),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.PYTHON]),
            reporting.Audience('developer'),
            reporting.ExternalLink(url, title),
            reporting.Remediation(hint='Please run "alternatives --set python /usr/bin/python3" after upgrade'),
            reporting.RelatedResource('package', 'python'),
            reporting.RelatedResource('package', 'python2'),
            reporting.RelatedResource('package', 'python3')
        ])



import os

import pytest

from leapp.libraries.common.config import mock_configs
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxCustom, SELinuxFacts, SELinuxModule, SELinuxModules, SELinuxRequestRPMs
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context

# compat module ensures compatibility with newer systems and is not part of testing
TEST_MODULES = [
    ["400", "mock1"],
    ["99", "mock1"],
    ["200", "mock1"],
    ["400", "mock2"],
    ["999", "mock3"],
    ["100", "compat"],
    ["200", "base_container"]
]

SEMANAGE_COMMANDS = [
    ['fcontext', '-t', 'httpd_sys_content_t', '"/web(/.*)?"'],
    ['fcontext', '-t', 'cgdcbxd_unit_file_t', '"cgdcbxd/(/.*)?"'],
    ['port', '-t', 'http_port_t', '-p', 'udp', '81'],
    ['permissive', 'abrt_t']
]

testmoduledir = "tests/mock_modules/"


def _run_cmd(cmd, logmsg="", split=False):
    try:
        return run(cmd, split=split).get("stdout", "")
    except CalledProcessError as e:
        # Only report issues when they are explicitly described.
        # This way expected failures are not reported.
        if logmsg:
            api.current_logger().warning("{}: {}".format(logmsg, e.stderr))
    return None


@pytest.fixture(scope="function")
def destructive_selinux_env():
    tests_dir = os.path.join(os.path.realpath(__file__).rsplit(os.path.sep, 2)[0], testmoduledir)
    for priority, module in TEST_MODULES:
        _run_cmd(["semodule", "-X", priority, "-i", os.path.join(tests_dir, module + ".cil")],
                 "Error installing mock module {} before test".format(module))

    for command in SEMANAGE_COMMANDS:
        _run_cmd(["semanage", command[0], "-a"] + command[1:],
                 "Error applying selinux customizations before test")

    yield

    for command in SEMANAGE_COMMANDS[:-1]:
        _run_cmd(["semanage", command[0], "-d"] + command[1:],
                 "Error removing selinux customizations after testing")

    for priority, module in reversed(TEST_MODULES + [["400", "permissive_abrt_t"]]):
        _run_cmd(["semodule", "-X", priority, "-r", module],
                 "Error removing selinux module {} after testing".format(module))


def find_module(selinuxmodules, name, priority):
    return next((module for module in selinuxmodules.modules
                if (module.name == name and module.priority == int(priority))), None)


def find_template(selinuxmodules, name, priority):
    return next((module for module in selinuxmodules.templates
                if (module.name == name and module.priority == int(priority))), None)


def find_semanage_rule(rules, rule):
    return next((r for r in rules if all(word in r for word in rule)), None)


@pytest.mark.skipif(os.getenv("DESTRUCTIVE_TESTING", False) in [False, "0"],
                    reason='Test disabled by default because it would modify the system')
def test_SELinuxContentScanner(current_actor_context, destructive_selinux_env):

    expected_data = {'policy': 'targeted',
                     'mls_enabled': True,
                     'enabled': True,
                     'runtime_mode': 'enforcing',
                     'static_mode': 'enforcing'}

    current_actor_context.feed(SELinuxFacts(**expected_data))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    modules = current_actor_context.consume(SELinuxModules)[0]
    assert modules
    # check that all modules installed during test setup where reported
    for priority, name in TEST_MODULES:
        if priority not in ('100', '200'):
            assert find_module(modules, name, priority)
    # check that udica template was reported
    assert find_template(modules, TEST_MODULES[-1][1], TEST_MODULES[-1][0])

    rpms = current_actor_context.consume(SELinuxRequestRPMs)[0]
    assert rpms

    # mock1 contains container related type
    assert "container-selinux" in rpms.to_install

    custom = current_actor_context.consume(SELinuxCustom)[0]
    assert custom
    # The second command contains removed type and should be discarded (in either upgrade path)
    assert find_semanage_rule(custom.removed, SEMANAGE_COMMANDS[1])
    # the rest of the commands should be reported (except for the last which will show up in modules)
    assert find_semanage_rule(custom.commands, SEMANAGE_COMMANDS[0])
    assert find_semanage_rule(custom.commands, SEMANAGE_COMMANDS[2])



from leapp.libraries.common import config
from leapp.libraries.stdlib import api
from leapp.models import CustomTargetRepository


def process():
    if not config.get_env('LEAPP_ENABLE_REPOS'):
        return
    api.current_logger().info('The --enablerepo option has been used,')
    for repoid in config.get_env('LEAPP_ENABLE_REPOS').split(','):
        api.produce(CustomTargetRepository(repoid=repoid))



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class FIPSInfo(Model):
    """
    Information about whether the source system has FIPS enabled.
    """
    topic = SystemInfoTopic

    is_enabled = fields.Boolean(default=False)
    """ Is fips enabled on the source system """



import os

from leapp.exceptions import StopActorExecution
from leapp.libraries.common import mdraid
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.utils.deprecation import deprecated


def has_grub(blk_dev):
    """
    Check whether GRUB is present on block device
    """
    try:
        blk = os.open(blk_dev, os.O_RDONLY)
        mbr = os.read(blk, 512)
    except OSError:
        api.current_logger().warning(
            'Could not read first sector of {} in order to identify the bootloader'.format(blk_dev)
        )
        raise StopActorExecution()
    os.close(blk)
    test = 'GRUB'
    if not isinstance(mbr, str):
        test = test.encode('utf-8')

    return test in mbr


def blk_dev_from_partition(partition):
    """
    Find parent device of /boot partition
    """
    try:
        result = run(['lsblk', '-spnlo', 'name', partition])
    except CalledProcessError:
        api.current_logger().warning(
            'Could not get parent device of {} partition'.format(partition)
        )
        raise StopActorExecution()
    # lsblk "-s" option prints dependencies in inverse order, so the parent device will always
    # be the last or the only device.
    # Command result example:
    # 'result', {'signal': 0, 'pid': 3872, 'exit_code': 0, 'stderr': u'', 'stdout': u'/dev/vda1\n/dev/vda\n'}
    return result['stdout'].strip().split()[-1]


def get_boot_partition():
    """
    Get /boot partition name.
    """
    try:
        # call grub2-probe to identify /boot partition
        result = run(['grub2-probe', '--target=device', '/boot'])
    except CalledProcessError:
        api.current_logger().warning(
            'Could not get name of underlying /boot partition'
        )
        raise StopActorExecution()
    except OSError:
        api.current_logger().warning(
            'Could not get name of underlying /boot partition:'
            ' grub2-probe is missing.'
            ' Possibly called on system that does not use GRUB2?'
        )
        raise StopActorExecution()
    boot_partition = result['stdout'].strip()
    api.current_logger().info('/boot is on {}'.format(boot_partition))
    return boot_partition


def get_grub_devices():
    """
    Get block devices where GRUB is located. We assume GRUB is on the same device
    as /boot partition is. In case that device is an md (Multiple Device) device, all
    of the component devices of such a device are considered.

    :return: Devices where GRUB is located
    :rtype: list
    """
    boot_device = get_boot_partition()
    devices = []
    if mdraid.is_mdraid_dev(boot_device):
        component_devs = mdraid.get_component_devices(boot_device)
        blk_devs = [blk_dev_from_partition(dev) for dev in component_devs]
        # remove duplicates as there might be raid on partitions on the same drive
        # even if that's very unusual
        devices = sorted(list(set(blk_devs)))
    else:
        devices.append(blk_dev_from_partition(boot_device))

    have_grub = [dev for dev in devices if has_grub(dev)]
    api.current_logger().info('GRUB is installed on {}'.format(",".join(have_grub)))
    return have_grub


@deprecated(since='2023-06-23', message='This function has been replaced by get_grub_devices')
def get_grub_device():
    """
    Get block device where GRUB is located. We assume GRUB is on the same device
    as /boot partition is.

    """
    boot_partition = get_boot_partition()
    grub_dev = blk_dev_from_partition(boot_partition)
    api.current_logger().info('GRUB is installed on {}'.format(grub_dev))
    # if has_grub(grub_dev):
    return grub_dev if has_grub(grub_dev) else None


def is_blscfg_enabled_in_defaultgrub(default_grub_msg):
    """
    Check if GRUB_ENABLE_BLSCFG is true in /etc/default/grub file
    """
    grub_options_lst = default_grub_msg.default_grub_info
    default_grub_options = {
        option.name: option.value.strip('"') for option in grub_options_lst
    }
    return bool(default_grub_options.get('GRUB_ENABLE_BLSCFG', '') == 'true')



from leapp.actors import Actor
from leapp.libraries.actor import adjustlocalrepos
from leapp.libraries.common import mounting
from leapp.libraries.stdlib import api
from leapp.models import (
    TargetOSInstallationImage,
    TargetUserSpaceInfo,
    TMPTargetRepositoriesFacts,
    UsedTargetRepositories
)
from leapp.tags import IPUWorkflowTag, TargetTransactionChecksPhaseTag
from leapp.utils.deprecation import suppress_deprecation


@suppress_deprecation(TMPTargetRepositoriesFacts)
class AdjustLocalRepos(Actor):
    """
    Adjust local repositories to the target user-space container.

    Changes the path of local file urls (starting with 'file://') for 'baseurl' and
    'mirrorlist' fields to the container space for the used repositories. This is
    done by prefixing host root mount bind ('/installroot') to the path. It ensures
    that the files will be accessible from the container and thus proper functionality
    of the local repository.
    """

    name = 'adjust_local_repos'
    consumes = (TargetOSInstallationImage,
                TargetUserSpaceInfo,
                TMPTargetRepositoriesFacts,  # deprecated
                UsedTargetRepositories)
    produces = ()
    tags = (IPUWorkflowTag, TargetTransactionChecksPhaseTag)

    def process(self):
        target_userspace_info = next(self.consume(TargetUserSpaceInfo), None)
        used_target_repos = next(self.consume(UsedTargetRepositories), None)
        target_repos_facts = next(self.consume(TMPTargetRepositoriesFacts), None)
        target_iso = next(self.consume(TargetOSInstallationImage), None)

        if not all([target_userspace_info, used_target_repos, target_repos_facts]):
            api.current_logger().error("Missing required information to proceed!")
            return

        target_repos_facts = target_repos_facts.repositories
        iso_repoids = set(repo.repoid for repo in target_iso.repositories) if target_iso else set()
        used_target_repoids = set(repo.repoid for repo in used_target_repos.repos)

        with mounting.NspawnActions(base_dir=target_userspace_info.path) as context:
            adjustlocalrepos.process(context, target_repos_facts, iso_repoids, used_target_repoids)



import pytest

from leapp.libraries.actor import scancustommodifications
from leapp.libraries.common.testutils import CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api

FILES_FROM_RPM = """
repos/system_upgrade/el8toel9/actors/xorgdrvfact/libraries/xorgdriverlib.py
repos/system_upgrade/el8toel9/actors/anotheractor/actor.py
repos/system_upgrade/el8toel9/files
"""

FILES_ON_SYSTEM = """
repos/system_upgrade/el8toel9/actors/xorgdrvfact/libraries/xorgdriverlib.py
repos/system_upgrade/el8toel9/actors/anotheractor/actor.py
repos/system_upgrade/el8toel9/files
/some/unrelated/to/leapp/file
repos/system_upgrade/el8toel9/files/file/that/should/not/be/there
repos/system_upgrade/el8toel9/actors/actor/that/should/not/be/there
"""

VERIFIED_FILES = """
.......T.    repos/system_upgrade/el8toel9/actors/xorgdrvfact/libraries/xorgdriverlib.py
S.5....T.    repos/system_upgrade/el8toel9/actors/anotheractor/actor.py
S.5....T.  c etc/leapp/files/pes-events.json
"""


@pytest.mark.parametrize('a_file,name', [
    ('repos/system_upgrade/el8toel9/actors/checkblacklistca/actor.py', 'checkblacklistca'),
    ('repos/system_upgrade/el7toel8/actors/checkmemcached/actor.py', 'check_memcached'),
    # actor library
    ('repos/system_upgrade/el7toel8/actors/checkmemcached/libraries/checkmemcached.py', 'check_memcached'),
    # actor file
    ('repos/system_upgrade/common/actors/createresumeservice/files/leapp_resume.service', 'create_systemd_service'),
    ('repos/system_upgrade/common/actors/commonleappdracutmodules/files/dracut/85sys-upgrade-redhat/do-upgrade.sh',
     'common_leapp_dracut_modules'),
    # not a library and not an actor file
    ('repos/system_upgrade/el7toel8/models/authselect.py', ''),
    ('repos/system_upgrade/common/files/rhel_upgrade.py', ''),
    # common library not tied to any actor
    ('repos/system_upgrade/common/libraries/mounting.py', ''),
    ('repos/system_upgrade/common/libraries/config/version.py', ''),
    ('repos/system_upgrade/common/libraries/multipathutil.py', ''),
    ('repos/system_upgrade/common/libraries/config/version.py', ''),
    ('repos/system_upgrade/common/libraries/dnfplugin.py', ''),
    ('repos/system_upgrade/common/libraries/testutils.py', ''),
    # the rest of false positives discovered by dkubek
    ('repos/system_upgrade/common/actors/setuptargetrepos/libraries/setuptargetrepos_repomap.py', 'setuptargetrepos'),
    ('repos/system_upgrade/el8toel9/actors/sssdfacts/libraries/sssdfacts8to9.py', 'sssd_facts_8to9'),
    ('repos/system_upgrade/el8toel9/actors/nisscanner/libraries/nisscan.py', 'nis_scanner'),
    ('repos/system_upgrade/common/actors/setuptargetrepos/libraries/setuptargetrepos_repomap.py', 'setuptargetrepos'),
    ('repos/system_upgrade/common/actors/repositoriesmapping/libraries/repositoriesmapping.py', 'repository_mapping'),
    ('repos/system_upgrade/common/actors/peseventsscanner/libraries/peseventsscanner_repomap.py',
     'pes_events_scanner')
])
def test_deduce_actor_name_from_file(a_file, name):
    assert scancustommodifications.deduce_actor_name(a_file) == name


def mocked__run_command(list_of_args, log_message, checked=True):
    if list_of_args == ['rpm', '-ql', 'leapp-upgrade-el8toel9']:
        # get source of truth
        return FILES_FROM_RPM.strip().split('\n')
    if list_of_args and list_of_args[0] == 'find':
        # listing files in directory
        return FILES_ON_SYSTEM.strip().split('\n')
    if list_of_args == ['rpm', '-V', '--nomtime', 'leapp-upgrade-el8toel9']:
        # checking authenticity
        return VERIFIED_FILES.strip().split('\n')
    return []


def test_check_for_modifications(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch='x86_64', src_ver='8.9', dst_ver='9.3'))
    monkeypatch.setattr(scancustommodifications, '_run_command', mocked__run_command)
    modifications = scancustommodifications.check_for_modifications('repository')
    modified = [m for m in modifications if m.type == 'modified']
    custom = [m for m in modifications if m.type == 'custom']
    configurations = [m for m in modifications if m.component == 'configuration']
    assert len(modified) == 3
    assert modified[0].filename == 'repos/system_upgrade/el8toel9/actors/xorgdrvfact/libraries/xorgdriverlib.py'
    assert modified[0].rpm_checks_str == '.......T.'
    assert len(custom) == 3
    assert custom[0].filename == '/some/unrelated/to/leapp/file'
    assert custom[0].rpm_checks_str == ''
    assert len(configurations) == 1
    assert configurations[0].filename == 'etc/leapp/files/pes-events.json'
    assert configurations[0].rpm_checks_str == 'S.5....T.'
